from django.template.context import RequestContext
from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404, redirect
from django.http import HttpResponseRedirect, HttpResponsePermanentRedirect
from django.contrib import messages
from django.core.urlresolvers import reverse

from tendenci.core.theme.shortcuts import themed_response as render_to_response
from tendenci.core.base.http import Http403
from tendenci.core.event_logs.models import EventLog
from tendenci.core.site_settings.utils import get_setting
from tendenci.core.exports.utils import run_export_task
from tendenci.core.perms.utils import has_perm, update_perms_and_save, get_notice_recipients, has_view_perm, get_query_filters
from tendenci.addons.help_files.models import HelpFile_Topics, Topic, HelpFile, HelpFileMigration, Request
from tendenci.addons.help_files.forms import RequestForm, HelpFileForm
from tendenci.apps.notifications import models as notification


def index(request, template_name="help_files/index.html"):
    """List all topics and all links"""
    topic_pks = []
    filters = get_query_filters(request.user, 'help_files.view_helpfile')

    # Access the join table and iterate over a dict to avoid
    # n+1 queries to get all of the correct topics.
    byhelpfile = {}
    for tp in HelpFile_Topics.objects.select_related('helpfile', 'topic').all():
        byhelpfile.setdefault(tp.helpfile.id, []).append(tp.topic)
    # Use stored lists
    for hf in HelpFile.objects.filter(filters).distinct():
        if byhelpfile.get(hf.id, ''):
            for topic in byhelpfile[hf.id]:
                topic_pks.append(topic.pk)

    topic_pks = sorted(list(set(topic_pks)))

    topics = Topic.objects.filter(pk__in=topic_pks)
    m = len(topics) / 2
    topics = topics[:m], topics[m:] # two columns
    most_viewed = HelpFile.objects.filter(filters).order_by('-view_totals').distinct()[:5]
    featured = HelpFile.objects.filter(filters).filter(is_featured=True).distinct()[:5]
    faq = HelpFile.objects.filter(filters).filter(is_faq=True).distinct()[:3]

    EventLog.objects.log()

    return render_to_response(template_name, locals(), 
        context_instance=RequestContext(request))


def search(request, template_name="help_files/search.html"):
    """ Help Files Search """
    query = request.GET.get('q', None)

    if get_setting('site', 'global', 'searchindex') and query:
        help_files = HelpFile.objects.search(query, user=request.user)
    else:
        filters = get_query_filters(request.user, 'help_files.view_helpfile')
        help_files = HelpFile.objects.filter(filters).distinct()
        if not request.user.is_anonymous():
            help_files = help_files.select_related()

    EventLog.objects.log()

    return render_to_response(template_name, {'help_files':help_files}, 
        context_instance=RequestContext(request))


def topic(request, id, template_name="help_files/topic.html"):
    """ List of topic help files """
    topic = get_object_or_404(Topic, pk=id)
    query = None

    filters = get_query_filters(request.user, 'help_files.view_helpfile')
    help_files = HelpFile.objects.filter(filters).filter(topics__in=[topic.pk]).distinct()
    if not request.user.is_anonymous():
        help_files = help_files.select_related()

    EventLog.objects.log(instance=topic)

    return render_to_response(template_name, {'topic':topic, 'help_files':help_files}, 
        context_instance=RequestContext(request))


def detail(request, slug, template_name="help_files/details.html"):
    """Help file details"""
    help_file = get_object_or_404(HelpFile, slug=slug)

    if has_view_perm(request.user, 'help_files.view_helpfile', help_file):
        HelpFile.objects.filter(pk=help_file.pk).update(view_totals=help_file.view_totals+1)
        EventLog.objects.log(instance=help_file)
        return render_to_response(template_name, {'help_file': help_file}, 
            context_instance=RequestContext(request))
    else:
        raise Http403

@login_required
def add(request, form_class=HelpFileForm, template_name="help_files/add.html"):
    if has_perm(request.user,'help_files.add_helpfile'):
        if request.method == "POST":
            form = form_class(request.POST, user=request.user)
            if form.is_valid():           
                help_file = form.save(commit=False)

                # add all permissions and save the model
                help_file = update_perms_and_save(request, form, help_file)
                form.save_m2m()

                messages.add_message(request, messages.SUCCESS, 'Successfully added %s' % help_file)
                
                # send notification to administrator(s) and module recipient(s)
                recipients = get_notice_recipients('module', 'help_files', 'helpfilerecipients')
                # if recipients and notification: 
#                     notification.send_emails(recipients,'help_file_added', {
#                         'object': help_file,
#                         'request': request,
#                     })

                return HttpResponseRedirect(reverse('help_file.details', args=[help_file.slug]))
        else:
            form = form_class(user=request.user)
           
        return render_to_response(template_name, {'form':form}, 
            context_instance=RequestContext(request))
    else:
        raise Http403

@login_required
def edit(request, id=None, form_class=HelpFileForm, template_name="help_files/edit.html"):
    help_file = get_object_or_404(HelpFile, pk=id)
    if has_perm(request.user,'help_files.change_helpfile', help_file):
        if request.method == "POST":
            form = form_class(request.POST, instance=help_file, user=request.user)
            if form.is_valid():           
                help_file = form.save(commit=False)

                # add all permissions and save the model
                help_file = update_perms_and_save(request, form, help_file)
                form.save_m2m()

                messages.add_message(request, messages.SUCCESS, 'Successfully edited %s' % help_file)
                
                # send notification to administrator(s) and module recipient(s)
                recipients = get_notice_recipients('module', 'help_files', 'helpfilerecipients')
                # if recipients and notification: 
#                     notification.send_emails(recipients,'help_file_added', {
#                         'object': help_file,
#                         'request': request,
#                     })

                return HttpResponseRedirect(reverse('help_file.details', args=[help_file.slug]))
        else:
            form = form_class(instance=help_file, user=request.user)
           
        return render_to_response(template_name, {'help_file': help_file, 'form':form}, 
            context_instance=RequestContext(request))
    else:
        raise Http403

def request_new(request, template_name="help_files/request_new.html"):
    "Request new file form"
    if request.method == 'POST':
        form = RequestForm(request.POST)
        if form.is_valid():
            instance = form.save()
            # send notification to administrators
            recipients = get_notice_recipients('module', 'help_files', 'helpfilerecipients')
            if recipients:
                if notification:
                    extra_context = {
                        'object': instance,
                        'request': request,
                    }
                    notification.send_emails(recipients,'help_file_requested', extra_context)
            messages.add_message(request, messages.INFO, 'Thanks for requesting a new help file!')
            EventLog.objects.log()
            return HttpResponseRedirect(reverse('help_files'))
    else:
        form = RequestForm()
        
    return render_to_response(template_name, {'form': form}, 
        context_instance=RequestContext(request))


def redirects(request, id):
    """
        Redirect old Tendenci 4 IDs to new Tendenci 5 slugs
    """
    try:
        help_file_migration = HelpFileMigration.objects.get(t4_id=id)
        try:
            help_file = HelpFile.objects.get(pk=help_file_migration.t5_id)
            return HttpResponsePermanentRedirect(help_file.get_absolute_url())
        except:
            return HttpResponsePermanentRedirect(reverse('help_files'))
    except:
        return HttpResponsePermanentRedirect(reverse('help_files'))
        

def requests(request, template_name="help_files/request_list.html"):
    """
        Display a list of help file requests
    """
    if not has_perm(request.user, 'help_files.change_request'):
        raise Http403
    
    requests = Request.objects.all()
    EventLog.objects.log()
    return render_to_response(template_name, {
        'requests': requests,
        }, context_instance=RequestContext(request))


@login_required
def export(request, template_name="help_files/export.html"):
    """Export Help Files"""
    
    if not request.user.is_superuser:
        raise Http403
    
    if request.method == 'POST':
        # initilize initial values
        file_name = "help_files.csv"
        fields = [
            'slug',
            'topics',
            'question',
            'answer',
            'level',
            'is_faq',
            'is_featured',
            'is_video',
            'syndicate',
            'view_totals',
        ]
        export_id = run_export_task('help_files', 'helpfile', fields)
        EventLog.objects.log()
        return redirect('export.status', export_id)
        
    return render_to_response(template_name, {
    }, context_instance=RequestContext(request))



import base64
from Crypto.Cipher import AES

from django.conf import settings


def encrypt(value):
    """Return the encrypted value of the setting.
    Uses the character '\0' as padding.
    """
    cipher = AES.new(settings.SITE_SETTINGS_KEY, AES.MODE_ECB)
    value = unicode(value)
    padding = cipher.block_size - len(value) % cipher.block_size
    for i in range(padding):
        value += '\0'
    ciphertext = cipher.encrypt(value)
    ciphertext = base64.b64encode(ciphertext) # make it database friendly
    return ciphertext

def decrypt(value):
    """Return the decrypted value of the setting.
    This removes the padding character '\0'
    """
    cipher = AES.new(settings.SITE_SETTINGS_KEY, AES.MODE_ECB)
    value = base64.b64decode(value) 
    value = cipher.decrypt(value)
    return value.replace('\0', '')

def test():
    """Check if original values and decrypted values
    will still be equal to each other.
    """
    from tendenci.core.site_settings.models import Setting
    s = Setting.objects.all()
    for x in s:
        code = encrypt(x.value)
        decode = decrypt(code)
        if x.value != decode:
            print "not equal!"



from django.core.management.base import BaseCommand

class Command(BaseCommand):
    """
    example: python manage.py events_init_end_dt
    """
    def handle(self, *event_ids, **options):
        from tendenci.addons.events.models import RegistrationConfiguration
        reg8n_configs = RegistrationConfiguration.objects.all()

        for config in reg8n_configs:
            config.end_dt = config.late_dt
            config.save()




# This module is deprecated as of 2012-09-07.
# It is no longer being imported or used.

from tendenci.apps.pluginmanager.settings import plugin_apps
from tendenci.apps.pluginmanager.urls import get_url_patterns



from django.template import Library, TemplateSyntaxError, Variable

from tendenci.core.base.template_tags import ListNode, parse_tag_kwargs
from tendenci.addons.jobs.models import Job
from django.db.models import Q

register = Library()


@register.inclusion_tag("jobs/options.html", takes_context=True)
def job_options(context, user, job):
    context.update({
        "opt_object": job,
        "user": user
    })
    return context


@register.inclusion_tag("jobs/nav.html", takes_context=True)
def job_nav(context, user, job=None):
    context.update({
        'nav_object': job,
        "user": user
    })
    return context


@register.inclusion_tag("jobs/search-form.html", takes_context=True)
def job_search(context):
    return context


@register.inclusion_tag("jobs/search-form.html", takes_context=True)
def my_job_search(context):
    context.update({
        'my_job': True
    })
    return context


@register.inclusion_tag("jobs/pricing-nav.html", takes_context=True)
def job_pricing_nav(context, user, job_pricing=None):
    context.update({
        'nav_object': job_pricing,
        "user": user
    })
    return context


@register.inclusion_tag("jobs/pricing-options.html", takes_context=True)
def job_pricing_options(context, user, job_pricing):
    context.update({
        "opt_object": job_pricing,
        "user": user
    })
    return context


@register.inclusion_tag("jobs/pricing-table.html", takes_context=True)
def job_pricing_table(context):
    from tendenci.addons.jobs.models import JobPricing
    job_pricings = JobPricing.objects.filter(status=True).order_by('duration')
    show_premium_price = False
    show_member_price = False
    premium_jp = JobPricing.objects.filter(status=True).filter(premium_price__gt=0)
    if premium_jp:
        show_premium_price = True
    member_jp = JobPricing.objects.filter(status=True).filter(show_member_pricing=True).filter(Q(premium_price_member__gt=0) | Q(regular_price_member__gt=0))
    if member_jp:
        show_member_price = True
    context.update({
        "job_pricings": job_pricings,
        'show_premium_price': show_premium_price,
        'show_member_price': show_member_price
    })
    return context


class ListJobNode(ListNode):
    model = Job
    perms = 'jobs.view_job'


@register.tag
def list_jobs(parser, token):
    """
    Used to pull a list of :model:`jobs.Job` items.

    Usage::

        {% list_jobs as [varname] [options] %}

    Be sure the [varname] has a specific name like ``jobs_sidebar`` or 
    ``jobs_list``. Options can be used as [option]=[value]. Wrap text values
    in quotes like ``tags="cool"``. Options include:
    
        ``limit``
           The number of items that are shown. **Default: 3**
        ``order``
           The order of the items. **Default: Newest Approved**
        ``user``
           Specify a user to only show public items to all. **Default: Viewing user**
        ``query``
           The text to search for items. Will not affect order.
        ``tags``
           The tags required on items to be included.
        ``random``
           Use this with a value of true to randomize the items included.

    Example::

        {% list_jobs as jobs_list limit=5 tags="cool" %}
        {% for job in jobs_list %}
            {{ job.title }}
        {% endfor %}
    """
    args, kwargs = [], {}
    bits = token.split_contents()
    context_var = bits[2]

    if len(bits) < 3:
        message = "'%s' tag requires at least 2 parameters" % bits[0]
        raise TemplateSyntaxError(message)

    if bits[1] != "as":
        message = "'%s' second argument must be 'as'" % bits[0]
        raise TemplateSyntaxError(message)

    kwargs = parse_tag_kwargs(bits)

    if 'order' not in kwargs:
        kwargs['order'] = '-post_dt'

    return ListJobNode(context_var, *args, **kwargs)



import simplejson as json
from StringIO import StringIO
from django.core.management import call_command
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    """
    Example: python manage.py settings_build_init_json.py
    """
    def handle(self, *args, **kwargs):
        """
        Build the initial settings json file via DB records
        """
        string_io = StringIO()
        call_command('dumpdata', 'site_settings', stdout=string_io)
        string_io.seek(0)
        s = json.loads(string_io.read())

        field_sets = [i['fields'] for i in s]

        json_dumps = json.dumps(field_sets, indent=4)

        f = open('settings.json', 'w')
        f.write(json_dumps)
        f.close()



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    depends_on = (
        ('files', '0001_initial'),
    )

    def forwards(self, orm):
        
        # Adding model 'Group'
        db.create_table('user_groups_group', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('allow_anonymous_view', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('allow_user_view', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('allow_member_view', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('allow_anonymous_edit', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('allow_user_edit', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('allow_member_edit', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('create_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('update_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('creator', self.gf('django.db.models.fields.related.ForeignKey')(related_name='user_groups_group_creator', to=orm['auth.User'])),
            ('creator_username', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('owner', self.gf('django.db.models.fields.related.ForeignKey')(related_name='user_groups_group_owner', to=orm['auth.User'])),
            ('owner_username', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('status', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('status_detail', self.gf('django.db.models.fields.CharField')(default='active', max_length=50)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
            ('slug', self.gf('tendenci.core.base.fields.SlugField')(unique=True, max_length=100, db_index=True)),
            ('guid', self.gf('django.db.models.fields.CharField')(max_length=40)),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('entity', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['entities.Entity'], null=True, blank=True)),
            ('type', self.gf('django.db.models.fields.CharField')(default='distribution', max_length=75, blank=True)),
            ('email_recipient', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('show_as_option', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('allow_self_add', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('allow_self_remove', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('description', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('auto_respond', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('auto_respond_template', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('auto_respond_priority', self.gf('django.db.models.fields.FloatField')(default=0, blank=True)),
            ('notes', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal('user_groups', ['Group'])

        # Adding M2M table for field permissions on 'Group'
        db.create_table('user_groups_group_permissions', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('group', models.ForeignKey(orm['user_groups.group'], null=False)),
            ('permission', models.ForeignKey(orm['auth.permission'], null=False))
        ))
        db.create_unique('user_groups_group_permissions', ['group_id', 'permission_id'])

        # Adding model 'GroupMembership'
        db.create_table('user_groups_groupmembership', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('group', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['user_groups.Group'])),
            ('member', self.gf('django.db.models.fields.related.ForeignKey')(related_name='group_member', to=orm['auth.User'])),
            ('role', self.gf('django.db.models.fields.CharField')(default='', max_length=255, blank=True)),
            ('sort_order', self.gf('django.db.models.fields.IntegerField')(default=0, blank=True)),
            ('creator_id', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('creator_username', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('owner_id', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('owner_username', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('status', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('status_detail', self.gf('django.db.models.fields.CharField')(default='active', max_length=50)),
            ('create_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('update_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal('user_groups', ['GroupMembership'])

        # Adding unique constraint on 'GroupMembership', fields ['group', 'member']
        db.create_unique('user_groups_groupmembership', ['group_id', 'member_id'])

        # Adding model 'ImportFile'
        db.create_table('user_groups_importfile', (
            ('file_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['files.File'], unique=True, primary_key=True)),
            ('group', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['user_groups.Group'])),
        ))
        db.send_create_signal('user_groups', ['ImportFile'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'GroupMembership', fields ['group', 'member']
        db.delete_unique('user_groups_groupmembership', ['group_id', 'member_id'])

        # Deleting model 'Group'
        db.delete_table('user_groups_group')

        # Removing M2M table for field permissions on 'Group'
        db.delete_table('user_groups_group_permissions')

        # Deleting model 'GroupMembership'
        db.delete_table('user_groups_groupmembership')

        # Deleting model 'ImportFile'
        db.delete_table('user_groups_importfile')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2012, 9, 19, 15, 19, 50, 385924)'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2012, 9, 19, 15, 19, 50, 385830)'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'entities.entity': {
            'Meta': {'object_name': 'Entity'},
            'admin_notes': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'allow_anonymous_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_anonymous_view': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_member_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_member_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'contact_name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'entity_creator'", 'to': "orm['auth.User']"}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'email': ('django.db.models.fields.CharField', [], {'max_length': '120', 'blank': 'True'}),
            'entity_name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'entity_parent_id': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'entity_type': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'fax': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'guid': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'notes': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'entity_owner'", 'to': "orm['auth.User']"}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'phone': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'summary': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'website': ('django.db.models.fields.CharField', [], {'max_length': '300', 'blank': 'True'})
        },
        'files.file': {
            'Meta': {'object_name': 'File'},
            'allow_anonymous_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_anonymous_view': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_member_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_member_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'files_file_creator'", 'to': "orm['auth.User']"}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '260'}),
            'guid': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_public': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'files_file_owner'", 'to': "orm['auth.User']"}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'tags': ('tagging.fields.TagField', [], {'null': 'True'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'user_groups.group': {
            'Meta': {'object_name': 'Group'},
            'allow_anonymous_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_anonymous_view': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_member_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_member_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_self_add': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_self_remove': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_user_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'auto_respond': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'auto_respond_priority': ('django.db.models.fields.FloatField', [], {'default': '0', 'blank': 'True'}),
            'auto_respond_template': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'user_groups_group_creator'", 'to': "orm['auth.User']"}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'email_recipient': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'entity': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['entities.Entity']", 'null': 'True', 'blank': 'True'}),
            'guid': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'members': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.User']", 'through': "orm['user_groups.GroupMembership']", 'symmetrical': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'notes': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'user_groups_group_owner'", 'to': "orm['auth.User']"}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'group_permissions'", 'blank': 'True', 'to': "orm['auth.Permission']"}),
            'show_as_option': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'slug': ('tendenci.core.base.fields.SlugField', [], {'unique': 'True', 'max_length': '100', 'db_index': 'True'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'type': ('django.db.models.fields.CharField', [], {'default': "'distribution'", 'max_length': '75', 'blank': 'True'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'user_groups.groupmembership': {
            'Meta': {'unique_together': "(('group', 'member'),)", 'object_name': 'GroupMembership'},
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator_id': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['user_groups.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'member': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'group_member'", 'to': "orm['auth.User']"}),
            'owner_id': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'role': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'user_groups.importfile': {
            'Meta': {'object_name': 'ImportFile', '_ormbases': ['files.File']},
            'file_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['files.File']", 'unique': 'True', 'primary_key': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['user_groups.Group']"})
        }
    }

    complete_apps = ['user_groups']



from datetime import datetime
from haystack import indexes
from haystack import site

from django.utils.html import strip_tags, strip_entities

from tendenci.addons.help_files.models import HelpFile
from tendenci.core.perms.indexes import TendenciBaseSearchIndex

class HelpFileIndex(TendenciBaseSearchIndex):
    question = indexes.CharField(model_attr='question')
    answer = indexes.CharField(model_attr='answer')
    syndicate = indexes.BooleanField(model_attr='syndicate')
    topic = indexes.MultiValueField()
    
    # RSS field
    can_syndicate = indexes.BooleanField()
    order = indexes.DateTimeField()
    
    def prepare_answer(self, obj):
        answer = obj.answer
        answer = strip_tags(answer)
        answer = strip_entities(answer)
        return answer
    
    def prepare_topic(self, obj):
        return [topic.pk for topic in obj.topics.all()]

    def prepare_can_syndicate(self, obj):
        return obj.allow_anonymous_view and obj.syndicate \
                and obj.status == 1  and obj.status_detail == 'active' \
                and obj.create_dt <= datetime.now()

    def prepare_order(self, obj):
        return obj.create_dt

site.register(HelpFile, HelpFileIndex)



from django.db import models
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType

class UnindexedItem(models.Model):
    content_type = models.ForeignKey(ContentType, db_index=True)
    object_id = models.PositiveIntegerField()
    create_dt = models.DateTimeField(auto_now_add=True)

    object = generic.GenericForeignKey('content_type', 'object_id')



from django.template import Library

register = Library()

@register.inclusion_tag("contacts/options.html", takes_context=True)
def contact_options(context, user, contact):
    context.update({
        "opt_object": contact,
        "user": user
    })
    return context

@register.inclusion_tag("contacts/nav.html", takes_context=True)
def contact_nav(context, user, contact=None):
    context.update({
        "nav_object" : contact,
        "user": user
    })
    return context

@register.inclusion_tag("contacts/search-form.html", takes_context=True)
def contact_search(context):
    return context


import os
import datetime
import re
from django.contrib.auth.models import User
from django.db import models
from django.http import HttpResponse
from django.db.models.fields import AutoField, FieldDoesNotExist
from django.utils.encoding import smart_str
from django.core.validators import email_re
from django.core.files.storage import default_storage

import xlrd
from xlwt import Workbook, XFStyle
from tendenci.apps.user_groups.models import GroupMembership
from tendenci.apps.profiles.models import Profile
from tendenci.core.base.utils import normalize_newline


# number rows to process per request
ROWS_TO_PROCESS = 10

# field.__class__.__name__
# DateTimeField
user_fields = [field for field in User._meta.fields if field.editable
               and (not field.__class__ == AutoField)]
user_field_names = [smart_str(field.name) for field in user_fields]
user_field_types = [field.__class__.__name__ for field in user_fields]
field_type_dict = dict(zip(user_field_names, user_field_types))
user_fields = None
user_field_types = None

profile_fields = [field for field in Profile._meta.fields if field.editable 
                  and (not field.__class__ == AutoField)]
profile_field_names = [smart_str(field.name) for field in profile_fields]
profile_field_types = [field.__class__.__name__ for field in profile_fields]
field_type_dict.update(dict(zip(profile_field_names, profile_field_types)))
profile_fields = None
profile_field_types = None


def handle_uploaded_file(f, file_path):
    destination = default_storage.open(file_path, 'wb+')
    for chunk in f.chunks():
        destination.write(chunk)
    destination.close()


def get_user_import_settings(request, id):
    d = {}

    if not id in request.session:
        return d

    d['file_name'] = (request.session[id]).get('file_name', u'')
    d['interactive'] = request.session[id].get('interactive', u'')
    d['override'] = request.session[id].get('override', u'')
    d['key'] = request.session[id].get('key', u'')
    d['group'] = request.session[id].get('group', u'')
    d['clear_group_membership'] = request.session[id].get(
                                'clear_group_membership', u''
                                )

    d['interactive'] = d.get('interactive') or False
    d['override'] = d.get('override') or False
    d['clear_group_membership'] = d.get('clear_group_membership') or False

    if d['override']:
        d['str_update'] = 'Override All Fields'
    else:
        d['str_update'] = 'Update Blank Fields'

    return d


def render_excel(filename, title_list, data_list, file_extension='.xls'):
    if file_extension == '.csv':
        str_out = ','.join(title_list)

        for row_item_list in data_list:
            for i in range(0, len(row_item_list)):
                if row_item_list[i]:
                    if isinstance(row_item_list[i], datetime.datetime):
                        row_item_list[i] = row_item_list[i].strftime(
                                    '%Y-%m-%d %H:%M:%S'
                                    )
                    elif isinstance(row_item_list[i], datetime.date):
                        row_item_list[i] = row_item_list[i].strftime(
                                '%Y-%m-%d'
                                )
                    elif isinstance(row_item_list[i], datetime.time):
                        row_item_list[i] = row_item_list[i].strftime(
                            '%H:%M:%S'
                            )

                # convert all to string; catches 0 (int)
                row_item_list[i] = '%s' % row_item_list[i]

            str_out += ','.join(row_item_list)

        content_type = "application/text"
    else:
        import StringIO
        output = StringIO.StringIO()
        export_wb = Workbook()
        export_sheet = export_wb.add_sheet('Sheet1')
        col_idx = 0
        for col_title in title_list:
            export_sheet.write(0, col_idx, "%s" % col_title)
            col_idx += 1
        row_idx = 1
        for row_item_list in data_list:
            col_idx = 0
            for cell_value in row_item_list:
                if cell_value:
                    cell_value_is_date = False
                    if isinstance(cell_value, datetime.datetime):
                        cell_value = xlrd.xldate.xldate_from_datetime_tuple((
                            cell_value.year, cell_value.month,
                            cell_value.day, cell_value.hour,
                            cell_value.minute,
                            cell_value.second), 0)
                        cell_value_is_date = True
                    elif isinstance(cell_value, datetime.date):
                        cell_value = xlrd.xldate.xldate_from_date_tuple((
                            cell_value.year,
                            cell_value.month,
                            cell_value.day), 0)
                        cell_value_is_date = True
                    elif isinstance(cell_value, datetime.time):
                        cell_value = xlrd.xldate.xldate_from_time_tuple((
                                cell_value.hour,
                                cell_value.minute,
                                cell_value.second))
                        cell_value_is_date = True
                    elif isinstance(cell_value, models.Model):
                        cell_value = str(cell_value)
                    if cell_value_is_date:
                        s = XFStyle()
                        s.num_format_str = 'M/D/YY'
                        export_sheet.write(row_idx, col_idx, cell_value, s)
                    else:
                        export_sheet.write(row_idx, col_idx, cell_value)
                col_idx += 1
            row_idx += 1
        export_wb.save(output)
        output.seek(0)
        str_out = output.getvalue()
        content_type = 'application/vnd.ms-excel'

    response = HttpResponse(str_out)
    response['Content-Type'] = content_type
    response['Content-Disposition'] = 'attachment; filename=' + filename
    return response


def user_import_process(request, setting_dict, preview=True, id=''):
    """
    This function processes each row and store the data
    in the user_object_dict. Then it updates the database
    if preview=False.
    """
    key_list = setting_dict['key'].split(',')
    # key(s)- user field(s) or profile fields(s)? that is import to identify
    key_user_list = [key for key in key_list if key in user_field_names]
    key_profile_list = [key for key in key_list if key in profile_field_names]

    setting_dict['total'] = request.session[id].get('total',  0)
    setting_dict['count_insert'] = 0
    setting_dict['count_update'] = 0
    setting_dict['count_invalid'] = 0

    data_dict_list = request.session[id].get('data_dict_list',  [])
    data_dict_list_len = len(data_dict_list)

    user_obj_list = []
    invalid_list = []

    start = 0
    if not preview:
        finish = start + ROWS_TO_PROCESS
        if finish > data_dict_list_len:
            finish = data_dict_list_len
    else:
        finish = data_dict_list_len

    for r in range(start, finish):
        user_object_dict = {}
        if not preview:
            user_import_dict = {}
        identity_user_dict = {}  # used to look up the User
        identity_profile_dict = {}  # used to look up the Profile
        missing_keys = []

        data_dict = data_dict_list[r]

        missing_keys = [key for key in data_dict.keys() \
                        if key in key_list \
                        and data_dict[key] == '']

        for key in data_dict.keys():
            user_object_dict[key] = data_dict[key]

            if key in key_list and data_dict[key] != '':
                if key in key_user_list:
                    identity_user_dict[key] = data_dict[key]
                if key in key_profile_list:
                    identity_profile_dict[key] = data_dict[key]

        user_object_dict['ROW_NUM'] = data_dict['ROW_NUM']

        if missing_keys:
            user_object_dict['ERROR'] = 'Missing key: %s.' % (
                            ', '.join(missing_keys))
            user_object_dict['IS_VALID'] = False
            setting_dict['count_invalid'] += 1
            if not preview:
                invalid_list.append({'ROW_NUM': user_object_dict['ROW_NUM'],
                                     'ERROR': user_object_dict['ERROR']})
        else:
            user_object_dict['IS_VALID'] = True

            # the keys could be the fields in both User and Profile tables
            user = get_user_by_key(identity_user_dict, identity_profile_dict)
            if user:
                if preview:
                    user_object_dict['ACTION'] = 'update'
                else:
                    user_import_dict['ACTION'] = 'update'
                setting_dict['count_update'] += 1

                if preview:
                    populate_user_dict(user, user_object_dict, setting_dict)
            else:
                #user = None
                if preview:
                    user_object_dict['ACTION'] = 'insert'
                else:
                    user_import_dict['ACTION'] = 'insert'
                setting_dict['count_insert'] += 1

            if not preview:
                user = do_user_import(request, user,
                                      user_object_dict,
                                      setting_dict)
                user_import_dict['user'] = user
                user_import_dict['ROW_NUM'] = user_object_dict['ROW_NUM']
                user_obj_list.append(user_import_dict)

        if preview:
            user_obj_list.append(user_object_dict)

    if not preview:
        if finish < data_dict_list_len:
            # not finished yet, store some data in the session
            count_insert = request.session[id].get('count_insert',  0) + \
            setting_dict['count_insert']
            count_update = request.session[id].get('count_update',  0) + \
            setting_dict['count_update']

            setting_dict['is_completed'] = False

            for r in range(start, finish):
                # remove those already processed rows
                data_dict_list.remove(data_dict_list[0])

            d = request.session[id]
            d.update({'is_completed': False,
                      'count_insert': count_insert,
                      'count_update': count_update,
                      'data_dict_list': data_dict_list})
            request.session[id] = d
        else:
            setting_dict['is_completed'] = True
            setting_dict['count_insert'] += request.session[id].get(
                                        'count_insert', 0)
            setting_dict['count_update'] += request.session[id].get(
                                            'count_update', 0)
            d = request.session[id]
            d.update({'is_completed': True})
            request.session[id] = d

    return user_obj_list, invalid_list


def get_user_by_key(identity_user_dict, identity_profile_dict):
    user = None

    if identity_user_dict and identity_profile_dict:
        users = User.objects.filter(**identity_user_dict)
        profiles = Profile.objects.filter(user__in=users).filter(
                        **identity_profile_dict
                        )
        if profiles:
            user = profiles[0].user
    elif identity_user_dict:
        users = User.objects.filter(**identity_user_dict)
        if users:
            user = users[0]
    else:
        profiles = Profile.objects.filter(**identity_profile_dict)
        if profiles:
            user = profiles[0].user

    return user


def do_user_import(request, user, user_object_dict, setting_dict):
    """
        the real work is here - do the insert or update
    """

    insert = not bool(user)  # insert or update
    user = user or User()  # existing or new user
    override = setting_dict['override']  # update ALL fields

    # insert/update user
    for field in user_field_names:
        if field == 'password' or field == 'username' or \
            (not insert and field in setting_dict['key']):
            continue
        if field in user_object_dict:
            if override:
                setattr(user, field, user_object_dict[field])
            else:
                # fill out the blank field only
                if getattr(user, field) == '':
                    setattr(user, field, user_object_dict[field])

    if insert:
        if 'username' in user_object_dict:  # set username
            user.username = user_object_dict['username']

        # generate if not username
        user.username = get_unique_username(user)

    if 'password' in user_object_dict and (insert or override):
        user.set_password(user_object_dict['password'])

    if not user.password:
        user.set_password(User.objects.make_random_password(length=8))

    user.is_active = bool(setting_dict['interactive'])

    if not bool(email_re.match(user.email)):
        user.email = ''  # if not valid; empty it out

    # loop through user properties; truncate at max_length
    for key, value in user.__dict__.items():
        try:
            max_length = User._meta.get_field_by_name(key)[0].max_length
        except FieldDoesNotExist:
            max_length = None
        if max_length:  # truncate per max_length field attribute
            setattr(user, key, value[:max_length])

    # username and email required
    if user.username and user.email:

        # insert/update record
        if insert:
            user.save(force_insert=True)
        else:
            user.save(force_update=True)

        try:  # get or create
            profile = user.get_profile()
        except Profile.DoesNotExist:
            profile = Profile.objects.create(user=user,
               creator=request.user,
               creator_username=request.user.username,
               owner=request.user,
               owner_username=request.user.username,
            )

        for field in profile_field_names:
            if field in user_object_dict:

                if override:
                    setattr(profile, field, user_object_dict[field])
                else:
                    # fill out the blank field only
                    if getattr(profile, field) == '':
                        setattr(profile, field, user_object_dict[field])

        profile.save()

        # add to group
        if setting_dict['group']:
            try:
                gm = GroupMembership.objects.get(group=setting_dict['group'],
                                                 member=user)
            except GroupMembership.DoesNotExist:
                gm = GroupMembership()
                gm.member = user
                gm.group = setting_dict['group']
                gm.creator_id = request.user.id
                gm.creator_username = request.user.username
                gm.owner_id = request.user.id
                gm.owner_username = request.user.username
                gm.status = 1
                gm.status_detail = 'active'
                gm.save()

    return user


def get_unique_username(user):

    if not user.username:
        if user.email:
            user.username = user.email

    if not user.username:
        if user.first_name and user.last_name:
            user.username = '%s%s' % (user.first_name[0], user.last_name)

    p = re.compile(r'[^\w.@+-]+', re.IGNORECASE)
    user.username = p.sub('', user.username)

    # check if this username already exists
    users = User.objects.filter(username__istartswith=user.username)

    if users:
        t_list = [u.username[len(user.username):] for u in users]
        num = 1
        while str(num) in t_list:
            num += 1
        user.username = '%s%s' % (user.username, str(num))

    return user.username


# populate user object to its dictionary, so we can display to the preview page
def populate_user_dict(user, user_dict, import_setting_list):
    if not 'first_name' in user_dict:
        user_dict['first_name'] = user.first_name
    if not 'last_name' in user_dict:
        user_dict['last_name'] = user.last_name
    if not 'email' in user_dict:
        user_dict['email'] = user.email
    if not 'username' in user_dict:
        user_dict['username'] = user.username
    if not import_setting_list['override']:
        if user.first_name:
            user_dict['first_name'] = user.first_name
        if user.last_name:
            user_dict['last_name'] = user.last_name
        if user.email:
            user_dict['email'] = user.email
        if user.username:
            user_dict['username'] = user.username
    if not user_dict['username']:
        user_dict['username'] = user.username


def get_header_list(file_path):
    if not os.path.isfile(file_path):
        raise NameError("%s is not a valid file." % file_path)

    header_list = []
    book = xlrd.open_workbook(file_path)
    sheet = book.sheet_by_index(0)
    for col in range(0, sheet.ncols):
        col_item = sheet.cell_value(rowx=0, colx=col)
        header_list.append(col_item)
    return header_list


def get_header_list_from_content(file_content, file_name):
    header_list = []
    if file_content and len(file_name) > 4:
        file_ext = file_name[-4:].lower()

        if file_ext == '.csv':
            line_return_index = file_content.find('\n')
            header_list = ((file_content[:line_return_index]
                            ).strip('\r')).split(',')
        else:
            book = xlrd.open_workbook(file_contents=file_content)
            nsheets = book.nsheets
            for i in range(0, nsheets):
                sh = book.sheet_by_index(i)
                for c in range(0, sh.ncols):
                    col_item = sh.cell_value(rowx=0, colx=c)
                    header_list.append(col_item)
    return header_list


def extract_from_excel(file_path):
    if not default_storage.exists(file_path):
        raise NameError("%s is not a valid file." % file_path)

    file_ext = (file_path[-4:]).lower()
    if file_ext != '.csv' and file_ext != '.xls':
        raise NameError(
    "%s is not a valid file type (should be either .csv or .xls)." % file_path
        )

    fields = []
    data_list = []

    if file_ext == '.csv':
        import csv
        import dateutil.parser as dparser

        normalize_newline(file_path)
        data = csv.reader(default_storage.open(file_path, 'rU'))

        # read the column header
        fields = data.next()
        fields = [smart_str(field) for field in fields]

        r = 1
        for row in data:
            item = dict(zip(fields, row))
            for key in item.keys():
                if key in field_type_dict and \
                field_type_dict[key] == 'DateTimeField':
                    item[key] = dparser.parser(item[key])
            item['ROW_NUM'] = r + 1
            data_list.append(item)
            r += 1
    else:
        book = xlrd.open_workbook(file_path)
        nsheets = book.nsheets
        nrows = book.sheet_by_index(0).nrows

        # get the fields from the first row
        for i in range(0, nsheets):
            sh = book.sheet_by_index(i)
            for c in range(0, sh.ncols):
                col_item = sh.cell_value(rowx=0, colx=c)
                fields.append(smart_str(col_item))

        # get the data - skip the first row
        for r in  range(1, nrows):
            row = []
            for i in range(0, nsheets):
                sh = book.sheet_by_index(i)
                for c in range(0, sh.ncols):
                    cell = sh.cell(r, c)
                    cell_value = cell.value
                    if cell.ctype == xlrd.XL_CELL_DATE:
                        date_tuple = xlrd.xldate_as_tuple(
                                        cell_value, book.datemode)
                        cell_value = datetime.date(date_tuple[0],
                                                   date_tuple[1],
                                                    date_tuple[2])
                    elif cell.ctype in (2, 3) \
                        and int(cell_value) == cell_value:
                        # so for zipcode 77079,
                        # we don't end up with 77079.0
                        cell_value = int(cell_value)
                    row.append(cell_value)

            item = dict(zip(fields, row))
            item['ROW_NUM'] = r + 1
            data_list.append(item)

    return data_list



# django
from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

class ThemeFileVersion(models.Model):
    file_name  = models.CharField(max_length=200, blank=False)
    content = models.TextField(max_length=150000,blank=True)
    relative_file_path = models.CharField(max_length=500, blank=True)
    author = models.ForeignKey(User, null=True, on_delete=models.SET_NULL)
    create_dt = models.DateTimeField(auto_now_add=True)
    
    class Meta:
        verbose_name        = _('theme archive')
        verbose_name_plural = _('theme archives')
        permissions = (("view_themefileversion","Can view theme version"),)



import sys
import time
import logging
import traceback

try:
    import cPickle as pickle
except ImportError:
    import pickle

from django.conf import settings
from django.core.mail import mail_admins
from django.contrib.auth.models import User
from django.contrib.sites.models import Site

from lockfile import FileLock, AlreadyLocked, LockTimeout

from tendenci.apps.notifications.models import NoticeQueueBatch
from tendenci.apps.notifications import models as notification

# lock timeout value. how long to wait for the lock to become available.
# default behavior is to never wait for the lock to be available.
LOCK_WAIT_TIMEOUT = getattr(settings, "NOTIFICATION_LOCK_WAIT_TIMEOUT", -1)

def send_all():
    lock = FileLock("send_notices")

    logging.debug("acquiring lock...")
    try:
        lock.acquire(LOCK_WAIT_TIMEOUT)
    except AlreadyLocked:
        logging.debug("lock already in place. quitting.")
        return
    except LockTimeout:
        logging.debug("waiting for the lock timed out. quitting.")
        return
    logging.debug("acquired.")

    batches, sent = 0, 0
    start_time = time.time()

    try:
        # nesting the try statement to be Python 2.4
        try:
            for queued_batch in NoticeQueueBatch.objects.all():
                notices = pickle.loads(str(queued_batch.pickled_data).decode("base64"))
                for user, label, extra_context, on_site in notices:
                    user = User.objects.get(pk=user)
                    logging.info("emitting notice to %s" % user)
                    # call this once per user to be atomic and allow for logging to
                    # accurately show how long each takes.
                    notification.send_now([user], label, extra_context, on_site)
                    sent += 1
                queued_batch.delete()
                batches += 1
        except:
            # get the exception
            exc_class, e, t = sys.exc_info()
            # email people
            current_site = Site.objects.get_current()
            subject = "[%s emit_notices] %r" % (current_site.name, e)
            message = "%s" % ("\n".join(traceback.format_exception(*sys.exc_info())),)
            mail_admins(subject, message, fail_silently=True)
            # log it as critical
            logging.critical("an exception occurred: %r" % e)
    finally:
        logging.debug("releasing lock...")
        lock.release()
        logging.debug("released.")
    
    logging.info("")
    logging.info("%s batches, %s sent" % (batches, sent,))
    logging.info("done in %.2f seconds" % (time.time() - start_time))



from datetime import datetime, timedelta
from django.utils.translation import ugettext_lazy as _
from django import forms

from tendenci.core.base.fields import SplitDateTimeField
from form_utils.forms import BetterForm

INITIAL_START_DT = datetime.now() - timedelta(weeks=4)
INITIAL_END_DT = datetime.now()


class EventsFilterForm(forms.Form):
    event_id = forms.IntegerField(required=False)
    ip = forms.CharField(max_length=16, required=False)
    user_id = forms.IntegerField(required=False)
    session_id = forms.CharField(max_length=40, required=False)

    def process_filter(self, queryset):
        cd = self.cleaned_data
        if cd['event_id']:
            queryset = queryset.filter(event_id=cd['event_id'])
        if cd['ip']:
            queryset = queryset.filter(user_ip_address=cd['ip'])
        if cd['user_id']:
            queryset = queryset.filter(user__pk=cd['user_id'])
        if cd['session_id']:
            queryset = queryset.filter(session_id=cd['session_id'])
        return queryset


class EventLogSearchForm(BetterForm):
    start_dt = SplitDateTimeField(
        label=_('Start Date/Time'),
        initial=INITIAL_START_DT,
        required=False
    )
    end_dt = SplitDateTimeField(
        label=_('End Date/Time'),
        initial=INITIAL_END_DT,
        required=False
    )
    request_method = forms.ChoiceField(
        required=False,
        choices=(('all', 'ALL',), ('post', 'POST',), ('get', 'GET',))
    )
    event_id = forms.IntegerField(required=False)
    source = forms.CharField(required=False)
    object_id = forms.CharField(required=False)
    user_ip_address = forms.CharField(required=False)
    user_id = forms.IntegerField(required=False)
    user_name = forms.CharField(required=False)
    session_id = forms.CharField(required=False)
    application = forms.CharField(required=False)
    action = forms.CharField(required=False)

    class Meta:
        fields = (
            'start_dt',
            'end_dt',
            'request_method',
            'event_id',
            'source',
            'object_id'
            'user_ip_address',
            'user_id',
            'user_name',
            'session_id',
            'application',
            'action',
            )

        fieldsets = [('',
            {
              'fields': ['start_dt',
                         'end_dt',
                         'request_method',
                         'event_id'
                         ],
              'legend': ''
              }),
            ('Advanced Options',
            {
              'fields': ['user_id',
                         'user_name',
                         'user_ip_address',
                         'session_id',
                         'source',
                         'object_id',
                         'application',
                         'action'
                         ],
              'legend': 'Advanced Options'
              }),
        ]



from django.db.models import Manager
from django.core.cache import cache
from django.conf import settings

from tendenci.core.robots.cache import CACHE_PRE_KEY, cache_all_robots


class RobotManager(Manager):
    def get_by_agent(self, user_agent):
        
        keys = [settings.CACHE_PRE_KEY, CACHE_PRE_KEY, 'all']
        key = '.'.join(keys)
        
        robots = cache.get(key)
        if not robots:
            cache_all_robots()
            robots = cache.get(key, [])
        
        # UnicodeDecodeError: 'ascii' codec can't decode byte 0xf3
        # http://stackoverflow.com/questions/2392732/sqlite-python-unicode-and-non-utf-data
        user_agent = unicode(user_agent, errors='ignore')

        for robot in robots:
            if robot.name.lower() in user_agent.lower():
                return robot
        return None



from django.template import Library

register = Library()


@register.inclusion_tag("event_logs/options.html", takes_context=True)
def event_log_options(context, user, event_log):
    context.update({
        "opt_object": event_log,
        "user": user
    })
    return context


@register.inclusion_tag("event_logs/nav.html", takes_context=True)
def event_log_nav(context, user, event_log=None):
    context.update({
        "nav_object": event_log,
        "user": user
    })
    return context


@register.inclusion_tag("event_logs/search-form.html", takes_context=True)
def event_log_search(context, search_form):
    context.update({
        "search_form": search_form,
    })
    return context



from django.db import models
from tendenci.apps.user_groups.models import Group
from tendenci.apps.user_groups.utils import get_default_group
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic

from tendenci.core.perms.models import TendenciBaseModel
from tendenci.core.perms.object_perms import ObjectPermission
from tendenci.core.base.fields import SlugField
from tinymce import models as tinymce_models
from managers import HelpFileManager

class Topic(models.Model):
    """Help topic"""
    title = models.CharField(max_length=255)
    content = models.TextField(null=True, blank=True)
    
    class Meta:
        ordering = ['title']
        
    @models.permalink
    def get_absolute_url(self):
        return ("help_files.topic", [self.pk])

    def __unicode__(self):
        return self.title

class HelpFile(TendenciBaseModel):
    """Question/Answer infromation"""
    LEVELS = ('basic', 'intermediate', 'advanced', 'expert')
    LEVEL_CHOICES = [(i,i) for i in LEVELS]

    slug = SlugField(_('URL Path'), unique=True)
    topics = models.ManyToManyField(Topic)
    question = models.CharField(max_length=500)
    answer = tinymce_models.HTMLField()
    level = models.CharField(choices=LEVEL_CHOICES, max_length=100, default='basic')
    is_faq = models.BooleanField()
    is_featured = models.BooleanField()
    is_video = models.BooleanField()
    syndicate = models.BooleanField(_('Include in RSS feed'), default=True)
    view_totals = models.PositiveIntegerField(default=0)

    group = models.ForeignKey(Group, null=True, default=get_default_group, on_delete=models.SET_NULL)
    perms = generic.GenericRelation(ObjectPermission,
                                          object_id_field="object_id",
                                          content_type_field="content_type")

    objects = HelpFileManager()

    class Meta:
        permissions = (("view_helpfile","Can view help file"),)

    @models.permalink
    def get_absolute_url(self):
        return ("help_file.details", [self.slug])
                
    def __unicode__(self):
        return self.question
    
    def level_is(self):
        "Template helper: {% if file.level_is.basic %}..."
        return dict([i, self.level==i] for i in HelpFile.LEVELS)


class HelpFile_Topics(models.Model):
    """
    This table is created automatically by the Many To 
    Many Relationship. It is added here to use in the 
    views to help optimize for certain queries.
    """
    helpfile = models.ForeignKey(HelpFile)
    topic = models.ForeignKey(Topic)

    class Meta:
        managed = False

class Request(models.Model):
    question = models.TextField()
    
    def __unicode__(self):
        return self.question
    

class HelpFileMigration(models.Model):
    """
        Unmanaged model to map old tendenci 4 id
        to the new tendenci 5 id
    """
    t4_id = models.IntegerField()
    t5_id = models.IntegerField()
    
    class Meta:
        managed = False
        db_table = 'mig_help_files_helpfile_t4_to_t5'
        
        
    


from django.conf.urls.defaults import patterns, url
from tendenci.addons.resumes.feeds import LatestEntriesFeed

urlpatterns = patterns('tendenci.addons.resumes.views',
    url(r'^$', 'search', name="resumes"),
    url(r'^search/$', 'search_redirect', name="resume.search"),
    url(r'^print-view/(?P<slug>[\w\-\/]+)/$', 'print_view', name="resume.print_view"),
    url(r'^add/$', 'add', name="resume.add"),
    url(r'^edit/(?P<id>\d+)/$', 'edit', name="resume.edit"),
    url(r'^edit/meta/(?P<id>\d+)/$', 'edit_meta', name="resume.edit.meta"),
    url(r'^delete/(?P<id>\d+)/$', 'delete', name="resume.delete"),
    url(r'^feed/$', LatestEntriesFeed(), name='resume.feed'),
    url(r'^pending/$', 'pending', name="resume.pending"),
    url(r'^approve/(?P<id>\d+)/$', 'approve', name="resume.approve"),
    url(r'^thank-you/$', 'thank_you', name="resume.thank_you"),
    url(r'^export/$', 'export', name="resume.export"),
    url(r'^(?P<slug>[\w\-\/]+)/file/$', 'resume_file', name="resume.resume_file"),
    url(r'^(?P<slug>[\w\-\/]+)/$', 'index', name="resume"),
)



import re
import os
from django.core.management import call_command
from django.core.management.base import BaseCommand, CommandError
from django.db import connections, transaction, DEFAULT_DB_ALIAS
from StringIO import StringIO
from django.conf import settings


class Command(BaseCommand):
    """
    Example: python manage.py set_theme thinksmart
    """

    def handle(self, theme_name, **options):
        """
        Set the website theme via theme name
        """
        from tendenci.core.site_settings.models import Setting

        try:
            setting = Setting.objects.get(
                name='theme',
                scope='module',
                scope_category='theme_editor',
            )
            setting.set_value(theme_name)
            setting.save()
            call_command('hide_settings', 'theme')
            call_command('update_settings', 'themes.%s' % theme_name.lstrip())
            call_command('clear_cache')
        except Setting.DoesNotExist:
            if int(options['verbosity']) > 0:
                print "We could not update the theme because the setting or theme is not available."
            




from django.core.management.base import BaseCommand
from django.contrib.auth.models import User

class Command(BaseCommand):

    def handle(self, *args, **options):
        """
        Converts subscribers to users, maintains group memberships, removes subscribers.
        """
        from tendenci.apps.subscribers.models import SubscriberData, GroupSubscription
        from tendenci.apps.user_groups.models import GroupMembership
        from tendenci.apps.profiles.models import Profile

        if GroupSubscription.objects.exists():
            for sub in GroupSubscription.objects.all():
                sub_email = sub.email

                if sub_email:        
                    if not User.objects.filter(email=sub_email).exists():
                        print "Creating new user for group subscription %s." % sub.id
                        if len(sub_email) <= 30:
                            username = sub_email
                        else:
                            username = sub_email.split("@")[0]
            
                        user = User(
                            username = username,
                            email = sub_email,
                            is_active = False
                        )
                        
                        if sub.subscriber_id:
                            user.first_name = sub.subscriber.get_first_name() 
                            user.last_name = sub.subscriber.get_last_name()
                            phone = sub.subscriber.get_phone_number()
                        else:
                            if SubscriberData.objects.filter(field_label="First Name").filter(subscription_id=sub.id):
                                user.first_name = SubscriberData.objects.filter(field_label="First Name").filter(subscription_id=sub.id)[0].value
                            if SubscriberData.objects.filter(field_label="Last Name").filter(subscription_id=sub.id):
                                user.last_name = SubscriberData.objects.filter(field_label="Last Name").filter(subscription_id=sub.id)[0].value
                            if SubscriberData.objects.filter(field_label__in=["Phone", "phone", "Phone Number", "phone number", "Home Phone", "Cell Phone"]).filter(subscription_id=sub.id):
                                phone = SubscriberData.objects.filter(field_label__in=["Phone", "phone", "Phone Number", "phone number", "Home Phone", "Cell Phone"]).filter(subscription_id=sub.id)[0].value
                            else:
                                phone = ''
                        
                        user.save()
                        
                        profile = Profile(
                            user = user,
                            creator = user,
                            creator_username = user.username,
                            owner = user,
                            owner_username = user.username,
                            phone = phone,
                            allow_anonymous_view = False
                        )
                        
                        profile.save()
            
                    else:
                        print "User with matching email found for group subscription %s."  % sub.id
            
                    user = User.objects.filter(email=sub_email).order_by('last_login')[0]

                    if not GroupMembership.objects.filter(group_id=sub.group_id).filter(member_id=user.id).exists():
                        print "Creating a group membership for user %s and group %s." % (user.id, sub.group_id)
                        gm = GroupMembership(
                            group_id = sub.group_id,
                            member_id = user.id,
                            role = "subscriber"
                        )
                        gm.save()
                    else:
                        print "Group membership already exists for user %s and group %s." % (user.id, sub.group_id)

                    print "Deleting group subscription %s." % sub.id
                    sub.delete()
                else:
                    print "No email field found for subscriber %s." % sub.id
        else:
            print "No group subscriptions exist."


from haystack import indexes
from haystack import site

from django.utils.html import strip_tags, strip_entities

from tendenci.apps.pages.models import Page
from tendenci.core.perms.indexes import TendenciBaseSearchIndex
from tendenci.core.categories.models import Category


class PageIndex(TendenciBaseSearchIndex):
    title = indexes.CharField(model_attr='title')
    content = indexes.CharField(model_attr='content')
    syndicate = indexes.BooleanField(model_attr='syndicate')

    # categories
    category = indexes.CharField()
    sub_category = indexes.CharField()

    # RSS fields
    can_syndicate = indexes.BooleanField()
    order = indexes.DateTimeField()

    def prepare_content(self, obj):
        content = obj.content
        content = strip_tags(content)
        content = strip_entities(content)
        return content

    def prepare_category(self, obj):
        category = Category.objects.get_for_object(obj, 'category')
        if category:
            return category.name
        return ''

    def prepare_sub_category(self, obj):
        category = Category.objects.get_for_object(obj, 'sub_category')
        if category:
            return category.name
        return ''

    def prepare_can_syndicate(self, obj):
        return obj.allow_anonymous_view and obj.syndicate and obj.status == 1  \
            and obj.status_detail == 'active'

    def prepare_order(self, obj):
        return obj.update_dt

site.register(Page, PageIndex)



from django.conf.urls.defaults import patterns, url, include

urlpatterns = patterns('tendenci.core.emails.views',
    url(r'^$', 'search', name="emails"),
    url(r'^search/$', 'search', name="email.search"),
    url(r'^(?P<id>\d+)/$', 'view', name="email.view"),
    url(r'^viewbody/(?P<id>\d+)/$', 'view', 
        {'template_name': 'emails/viewbody.html'}, 
        name="email.viewbody"),
    url(r'^add/$', 'add', name="email.add"),
    url(r'^edit/(?P<id>\d+)/$', 'edit', name="email.edit"),
    url(r'^delete/(?P<id>\d+)/$', 'delete', name="email.delete"),
    url(r'^amazon_ses/$', 'amazon_ses_index', 
        name="email.amazon_ses_index"),
    url(r'^amazon_ses/verify_email/$', 'amazon_ses_verify_email', 
        name="email.amazon_ses_verify_email"),
    url(r'^amazon_ses/list_verified_emails/$', 'amazon_ses_list_verified_emails', 
        name="email.amazon_ses_list_verified_emails"),
    url(r'^amazon_ses/send_quota/$', 'amazon_ses_send_quota', 
        name="email.amazon_ses_send_quota"),
    (r'^blocks/', include('tendenci.core.email_blocks.urls')),
)


from django import forms
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe

class UseCustomRegWidget(forms.MultiWidget):
    """This widget is for three fields on event add/edit under Registration:
            use_custom_reg_form
            reg_form
            bind_reg_form_to_conf_only
    """
    def __init__(self, attrs=None, reg_form_choices=None, event_id=None):
        self.attrs = attrs
        self.reg_form_choices = reg_form_choices
        self.event_id = event_id
        if not self.attrs:
            self.attrs = {'id':'use_custom_reg'}
        self.widgets = (forms.CheckboxInput(),
                        forms.Select(), 
                        forms.RadioSelect())
        super(UseCustomRegWidget, self).__init__(self.widgets, attrs)
        
    def render(self, name, value, attrs=None):
        if not isinstance(value, list):
            value = self.decompress(value)
            
        final_attrs = self.build_attrs(attrs)
        id_ = final_attrs.get('id', None)
        use_custom_reg_form_widget = self.widgets[0]
        rendered_use_custom_reg_form = self.render_widget(
                                    use_custom_reg_form_widget,
                                    name, value, final_attrs, 
                                    0, id_)                                             
        
        reg_form_widget = self.widgets[1]
        reg_form_widget.choices = self.reg_form_choices
        #reg_form_widget.attrs = {'size':'8'}
        rendered_reg_form = '<div style="margin-left:2em;">%s</div>' % \
                            self.render_widget(reg_form_widget, 
                                                name, value, final_attrs, 
                                                1, id_)
        bind_reg_form_to_conf_only_widget = self.widgets[2]
        choices=(('1', mark_safe('Use one form for all pricings %s' \
                                 % rendered_reg_form)),)
        bind_reg_form_to_conf_only_widget.choices = choices
        rendered_bind_reg_form_to_conf_only = self.render_widget(
                                    bind_reg_form_to_conf_only_widget,
                                    name, value, final_attrs, 
                                    2, id_) 
        rendered_bind_reg_form_to_conf_only = rendered_bind_reg_form_to_conf_only.replace(
                               '%s</label>' % rendered_reg_form, "</label>%s" % rendered_reg_form           
                                                                                          )
        if self.event_id:
            manage_custom_reg_link = """
                            <div style="margin: 1em 0 0 2em;"><a href="%s" target="_blank">Manage Custom Registration Form</a></div>
                                    """ % reverse('event.event_custom_reg_form_list', args=[self.event_id])
        else:
            manage_custom_reg_link = ''
            
        output_html = """
                      <div id="use-custom_reg-box">
                          <div id="use-custom-reg-checkbox">%s <label for="id_%s_%s">Use Custom Registration Form</label></div>
                          
                          <div id="one-or-separate-form">%s</div>
                          %s
                      </div>
                    """ % (
                           rendered_use_custom_reg_form, name, '0', 
                           rendered_bind_reg_form_to_conf_only,
                           manage_custom_reg_link
                           )
        return mark_safe(output_html)
    
    def render_widget(self, widget, name, value, attrs, index=0, id=None):
        i = index
        id_ = id
        if value:
            try:
                widget_value = value[i]
            except IndexError:
                self.fields['use_reg_form'].initial = None
        else:
            widget_value = None
        if id_:
            final_attrs = dict(attrs, id='%s_%s' % (id_, i))

        return widget.render(name+'_%s' %i, widget_value, final_attrs)
        
    
    def decompress(self, value):
        if value:
            data_list = value.split(',')
            if data_list[0] == '1':
                data_list[0] = 'on'
            return data_list
        return None
        



from django.contrib.auth.models import User
from django.test import TestCase

from tendenci.apps.forms_builder.forms.models import Form, FIELD_CHOICES, \
    STATUS_DRAFT, STATUS_PUBLISHED


class Tests(TestCase):

    def test_form_fields(self):
        """
        Simple 200 status check against rendering and posting to forms with 
        both optional and required fields.
        """
        for required in (True, False):
            form = Form.objects.create(title="Test", status=STATUS_PUBLISHED)
            for field in FIELD_CHOICES:
                form.fields.create(label=field[0], field_type=field[0], 
                    required=required, visible=True)
            response = self.client.get(form.get_absolute_url())
            self.assertEqual(response.status_code, 200)
            fields = form.fields.visible()
            data = dict([("field_%s" % f.id, "test") for f in fields])
            response = self.client.post(form.get_absolute_url(), data=data)
            self.assertEqual(response.status_code, 200)

    def test_draft_form(self):
        """
        Test that a form with draft status is only visible to staff.
        """
        username = "test"
        password = "test"
        User.objects.create_superuser(username, "", password)
        self.client.logout()
        draft = Form.objects.create(title="Draft", status=STATUS_DRAFT)
        response = self.client.get(draft.get_absolute_url())
        self.assertEqual(response.status_code, 404)
        self.client.login(username=username, password=password)
        response = self.client.get(draft.get_absolute_url())
        self.assertEqual(response.status_code, 200)




import os
import zipfile

from django.conf import settings

from celery.task import Task
from celery.registry import tasks

class ZipPhotoSetTask(Task):

    def run(self, photo_set, **kwargs):
        """
        Compile all photos of a photo set into a single zip file.
        """
        
         #create zip files directory if it doesn't already exist
        try:
            os.makedirs(os.path.join(settings.MEDIA_ROOT, 'zip_files'))
        except OSError:
            pass
        
        zfile = zipfile.ZipFile(os.path.join(settings.MEDIA_ROOT, 'zip_files', "set_%s.zip" % photo_set.id), 'w')
        for image in photo_set.image_set.all():
            try:
                zfile.write(image.image.path, os.path.basename(image.image.name))
            except OSError:
                # skip missing files
                pass
        zfile.close
        
        return os.path.join(settings.MEDIA_URL, 'zip_files', "set_%s.zip" % photo_set.id)



from django.db import models
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User

from tendenci.core.perms.managers import ObjectPermissionManager
from tendenci.apps.user_groups.models import Group


class ObjectPermission(models.Model):
    """
    Object level permissions

    Don't move this model into the models.py
    because it will cause circular references
    all over the place. Please leave it here.
    """
    user = models.ForeignKey(User, null=True)
    group = models.ForeignKey(Group, null=True)
    content_type = models.ForeignKey(ContentType)
    codename = models.CharField(max_length=255)
    object_id = models.IntegerField()
    create_dt = models.DateTimeField(auto_now_add=True)
    object = generic.GenericForeignKey('content_type', 'object_id')

    objects = ObjectPermissionManager()



from django.conf.urls.defaults import patterns, url

from tendenci.core.files.signals import init_signals


init_signals()

urlpatterns = patterns('tendenci.core.files.views',
    url(r'^$', 'search', name="files"),
    url(r'^(?P<id>\d+)/$', 'details', name="file"),
    url(r'^(?P<id>\d+)/(?P<download>(download)?)/$', 'details', name="file"),
    url(r'^(?P<id>\d+)/(?P<size>\d*x\d*)/$', 'details', name="file"),
    url(r'^(?P<id>\d+)/(?P<size>\d*x\d*)/(?P<constrain>constrain)/$', 'details', name="file"),
    url(r'^(?P<id>\d+)/(?P<size>\d*x\d*)/(?P<download>download)/$', 'details', name="file"),
    url(r'^(?P<id>\d+)/(?P<size>\d*x\d*)/(?P<download>download)/(?P<constrain>constrain)/$', 'details', name="file"),

    # crop and quality
    url(r'^(?P<id>\d+)/(?P<size>\d+x\d+)/(?P<crop>crop)/$', 'details', name="file"),
    url(r'^(?P<id>\d+)/(?P<size>\d+x\d+)/(?P<quality>\d+)/$', 'details', name="file"),
    url(r'^(?P<id>\d+)/(?P<size>\d+x\d+)/(?P<crop>crop)/(?P<quality>\d+)/$', 'details', name="file"),

    url(r'^search/$', 'search_redirect', name="file.search"),
    url(r'^add/$', 'add', name="file.add"),
    url(r'^bulk-add/$', 'bulk_add', name="file.bulk_add"),
    url(r'^edit/(?P<id>\d+)/$', 'edit', name="file.edit"),
    url(r'^delete/(?P<id>\d+)/$', 'delete', name="file.delete"),

    url(r'^tinymce/$', 'tinymce', name="file.tinymce"),
    url(r'^tinymce/template/(?P<id>\d+)/$', 'tinymce_upload_template'),
    url(r'^swfupload/$', 'swfupload', name="file.swfupload"),

    url(r'^reports/most-viewed/$', 'report_most_viewed', name="file.report_most_viewed"),
)



import datetime

from django.conf import settings
from django.core.urlresolvers import reverse
from django.core.mail import EmailMessage
from django.db.models import Q
from django.template import RequestContext
from django.shortcuts import get_object_or_404, redirect
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect, HttpResponse
from django.forms.models import inlineformset_factory
from django.contrib import messages
from django.utils.encoding import smart_str
from django.template.defaultfilters import yesno
from django.core.files.storage import default_storage
from django.template.loader import get_template

from tendenci.core.theme.shortcuts import themed_response as render_to_response
from tendenci.core.base.http import Http403
from tendenci.core.base.utils import check_template, template_exists
from tendenci.core.perms.utils import (has_perm, update_perms_and_save,
    get_query_filters, has_view_perm)
from tendenci.core.event_logs.models import EventLog
from tendenci.core.site_settings.utils import get_setting
from tendenci.apps.invoices.models import Invoice
from tendenci.addons.recurring_payments.models import RecurringPayment
from tendenci.core.exports.utils import run_export_task

from tendenci.apps.forms_builder.forms.forms import (FormForForm, FormForm, FormForField,
    PricingForm, BillingForm)
from tendenci.apps.forms_builder.forms.models import Form, Field, FormEntry, Pricing
from tendenci.apps.forms_builder.forms.utils import (generate_admin_email_body,
    generate_submitter_email_body, generate_email_subject,
    make_invoice_for_entry, update_invoice_for_entry)
from tendenci.apps.forms_builder.forms.formsets import BaseFieldFormSet

@login_required
def add(request, form_class=FormForm, template_name="forms/add.html"):
    if not has_perm(request.user,'forms.add_form'):
        raise Http403

    PricingFormSet = inlineformset_factory(Form, Pricing, form=PricingForm, extra=2, can_delete=False)

    formset = PricingFormSet()
    if request.method == "POST":
        form = form_class(request.POST, user=request.user)
        if form.is_valid():
            form_instance = form.save(commit=False)
            # save form and associated pricings
            form_instance = update_perms_and_save(request, form, form_instance)
            formset = PricingFormSet(request.POST, instance=form_instance)
            if formset.is_valid():
                # update_perms_and_save does not appear to consider ManyToManyFields
                for method in form.cleaned_data['payment_methods']:
                    form_instance.payment_methods.add(method)

                formset.save()

                messages.add_message(request, messages.SUCCESS, 'Successfully added %s' % form_instance)
                return HttpResponseRedirect(reverse('form_field_update', args=[form_instance.pk]))
    else:
        form = form_class(user=request.user)

    return render_to_response(template_name, {
        'form':form,
        'formset':formset,
    }, context_instance=RequestContext(request))


def edit(request, id, form_class=FormForm, template_name="forms/edit.html"):
    form_instance = get_object_or_404(Form, pk=id)

    if not has_perm(request.user,'forms.change_form',form_instance):
        raise Http403

    PricingFormSet = inlineformset_factory(Form, Pricing, form=PricingForm, extra=2)

    if request.method == "POST":
        form = form_class(request.POST, instance=form_instance, user=request.user)
        if form_instance.recurring_payment:
            formset = RecurringPaymentFormSet(request.POST, instance=form_instance)
        else:
            formset = PricingFormSet(request.POST, instance=form_instance)
        if form.is_valid() and formset.is_valid():
            form_instance = form.save(commit=False)
            form_instance = update_perms_and_save(request, form, form_instance)

            form.save_m2m()  # save payment methods
            formset.save()  # save price options

            # remove all pricings if no custom_payment form
            if not form.cleaned_data['custom_payment']:
                form_instance.pricing_set.all().delete()

            messages.add_message(request, messages.SUCCESS, 'Successfully edited %s' % form_instance)
            return HttpResponseRedirect(reverse('form_field_update', args=[form_instance.pk]))
    else:
        form = form_class(instance=form_instance, user=request.user)
        if form_instance.recurring_payment:
            formset = RecurringPaymentFormSet(instance=form_instance)
        else:
            formset = PricingFormSet(instance=form_instance)
    return render_to_response(template_name, {
        'form':form,
        'formset':formset,
        'form_instance':form_instance,
        },context_instance=RequestContext(request))


@login_required
def update_fields(request, id, template_name="forms/update_fields.html"):
    form_instance = get_object_or_404(Form, id=id)

    if not has_perm(request.user,'forms.add_form',form_instance):
        raise Http403

    form_class=inlineformset_factory(Form, Field, form=FormForField, formset=BaseFieldFormSet, extra=3)
    form_class._orderings = 'position'

    if request.method == "POST":
        form = form_class(request.POST, instance=form_instance, queryset=form_instance.fields.all().order_by('position'))
        if form.is_valid():
            form.save()
            EventLog.objects.log()
            messages.add_message(request, messages.SUCCESS, 'Successfully updated %s' % form_instance)
            return redirect('form_detail', form_instance.slug)
    else:
        form = form_class(instance=form_instance, queryset=form_instance.fields.all().order_by('position'))

    return render_to_response(template_name, {'form':form, 'form_instance':form_instance},
        context_instance=RequestContext(request))


@login_required
def delete(request, id, template_name="forms/delete.html"):
    form_instance = get_object_or_404(Form, pk=id)

    # check permission
    if not has_perm(request.user,'forms.delete_form',form_instance):
        raise Http403

    if request.method == "POST":
        messages.add_message(request, messages.SUCCESS, 'Successfully deleted %s' % form_instance)

        form_instance.delete()
        return HttpResponseRedirect(reverse('forms'))

    return render_to_response(template_name, {'form': form_instance},
        context_instance=RequestContext(request))

@login_required
def copy(request, id):
    """
    Copies a form_instance and all the fields related to it.
    """
    form_instance = get_object_or_404(Form, pk=id)

    # check permission
    if not (has_perm(request.user,'forms.add_form',form_instance) and
        has_perm(request.user,'forms.change_form',form_instance)):
            raise Http403

    # create a new slug
    slug = form_instance.slug
    if len(slug.rsplit('-', 1)) > 1 and slug.rsplit('-', 1)[1].isdigit():
        slug = slug.rsplit('-', 1)[0]
    i = 1
    while True:
        if i > 0:
            if i > 1:
                slug = slug.rsplit("-", 1)[0]
            slug = "%s-%s" % (slug, i)
        match = Form.objects.filter(slug=slug)
        if not match:
            break
        i += 1

    # copy the form
    new_form = Form.objects.create(
        title = form_instance.title,
        slug = slug,
        intro = form_instance.intro,
        response = form_instance.response,
        email_text = form_instance.email_text,
        subject_template = form_instance.subject_template,
        send_email = form_instance.send_email,
        email_from = form_instance.email_from,
        email_copies = form_instance.email_copies,
        completion_url = form_instance.completion_url,
        template = form_instance.template,
        allow_anonymous_view = form_instance.allow_anonymous_view,
        allow_user_view = form_instance.allow_user_view,
        allow_member_view = form_instance.allow_member_view,
        allow_user_edit = form_instance.allow_user_edit,
        allow_member_edit = form_instance.allow_member_edit,
        creator = request.user,
        creator_username = request.user.username,
        owner = request.user,
        owner_username = request.user.username,
        status = True,
        status_detail = 'draft',
        )

    # copy form fields
    for field in form_instance.fields.all():
        Field.objects.create(
            form = new_form,
            label = field.label,
            field_type = field.field_type,
            field_function = field.field_function,
            function_params = field.function_params,
            required = field.required,
            visible = field.visible,
            choices = field.choices,
            position = field.position,
            default = field.default,
            )

    EventLog.objects.log(instance=form_instance)
    messages.add_message(request, messages.SUCCESS, 'Successfully added %s' % new_form)
    return redirect('form_edit', new_form.pk)

@login_required
def entries(request, id, template_name="forms/entries.html"):
    form = get_object_or_404(Form, pk=id)

    if not has_perm(request.user,'forms.change_form',form):
        raise Http403

    entries = form.entries.all()

    EventLog.objects.log(instance=form)

    return render_to_response(template_name, {'form':form,'entries': entries},
        context_instance=RequestContext(request))


@login_required
def entry_delete(request, id, template_name="forms/entry_delete.html"):
    entry = get_object_or_404(FormEntry, pk=id)

    # check permission
    if not has_perm(request.user,'forms.delete_form',entry.form):
        raise Http403

    if request.method == "POST":
        messages.add_message(request, messages.SUCCESS, 'Successfully deleted entry %s' % entry)
        entry.delete()
        return HttpResponseRedirect(reverse('forms'))

    return render_to_response(template_name, {'entry': entry},
        context_instance=RequestContext(request))

@login_required
def entry_detail(request, id, template_name="forms/entry_detail.html"):
    entry = get_object_or_404(FormEntry, pk=id)

    # check permission
    if not has_perm(request.user,'forms.change_form',entry.form):
        raise Http403

    return render_to_response(template_name, {'entry':entry},
        context_instance=RequestContext(request))


def entries_export(request, id):
    form_instance = get_object_or_404(Form, pk=id)

    # check permission
    if not has_perm(request.user,'forms.change_form',form_instance):
        raise Http403

    EventLog.objects.log(instance=form_instance)

    entries = form_instance.entries.all()

    if entries:
        import csv
        import zipfile
        from os.path import join
        from os import unlink
        from time import time
        from tempfile import NamedTemporaryFile

        response = HttpResponse(mimetype='text/csv')
        response['Content-Disposition'] = 'attachment; filename=export_entries_%d.csv' % time()
        headers = []
        has_files = False

        # check for a field field
        for entry in entries:
            for field in entry.fields.all():
                if field.field.field_type == 'FileField':
                    has_files = True

        # if the object hase file store the csv elsewhere
        # so that we can zip the files
        if has_files:
            temp_csv = NamedTemporaryFile(mode='w', delete=False)
            temp_zip = NamedTemporaryFile(mode='wb', delete=False)
            writer = csv.writer(temp_csv, delimiter=',')
            zip = zipfile.ZipFile(temp_zip, 'w', compression=zipfile.ZIP_DEFLATED)
        else:
            writer = csv.writer(response, delimiter=',')

        # get the header for headers for the csv
        headers.append('submitted on')
        for field in entries[0].fields.all():
            headers.append(smart_str(field.field.label))
        writer.writerow(headers)

        # write out the values
        for entry in entries:
            values = []
            values.append(entry.entry_time)
            for field in entry.fields.all():
                if has_files and field.field.field_type == 'FileField':
                    archive_name = join('files',field.value)
                    if hasattr(settings, 'USE_S3_STORAGE') and settings.USE_S3_STORAGE:
                        file_path = field.value
                        try:
                            # TODO: for large files, we may need to copy down
                            # the files before adding them to the zip file.
                            zip.writestr(archive_name, default_storage.open(file_path).read())
                        except IOError:
                            pass
                    else:
                        file_path = join(settings.MEDIA_ROOT,field.value)
                        zip.write(file_path, archive_name, zipfile.ZIP_DEFLATED)

                if field.field.field_type == 'BooleanField':
                    values.append(yesno(smart_str(field.value)))
                else:
                    values.append(smart_str(field.value))
            writer.writerow(values)

        # add the csv file to the zip, close it, and set the response
        if has_files:
            # add the csv file and close it all out
            temp_csv.close()
            zip.write(temp_csv.name, 'entries.csv', zipfile.ZIP_DEFLATED)
            zip.close()
            temp_zip.close()

            # set the response for the zip files
            response = HttpResponse(open(temp_zip.name,'rb'), mimetype='application/zip')
            response['Content-Disposition'] = 'attachment; filename=export_entries_%d.zip' % time()

            # remove the temporary files
            unlink(temp_zip.name)
            unlink(temp_csv.name)
    else:
        # blank csv document
        response = HttpResponse(mimetype='text/csv')
        response['Content-Disposition'] = 'attachment; filename=export_entries_%d.csv' % time()
        writer = csv.writer(response, delimiter=',')

    return response


def search(request, template_name="forms/search.html"):
    if not has_perm(request.user,'forms.view_form'):
        raise Http403

    filters = get_query_filters(request.user, 'forms.view_form')
    forms = Form.objects.filter(filters).distinct()
    query = request.GET.get('q', None)
    if query:
        forms = forms.filter(Q(title__icontains=query)|Q(intro__icontains=query)|Q(response__icontains=query))

    forms = forms.order_by('-pk')

    EventLog.objects.log()

    return render_to_response(template_name, {'forms':forms},
        context_instance=RequestContext(request))


def form_detail(request, slug, template="forms/form_detail.html"):
    """
    Display a built form and handle submission.
    """
    published = Form.objects.published(for_user=request.user)
    form = get_object_or_404(published, slug=slug)

    if not has_view_perm(request.user,'forms.view_form',form):
        raise Http403

    # If form has a recurring payment, make sure the user is logged in
    if form.recurring_payment and request.user.is_anonymous():
        response = redirect('auth_login')
        response['Location'] += '?next=%s' % form.get_absolute_url()
        return response

    form_for_form = FormForForm(form, request.user, request.POST or None, request.FILES or None)

    for field in form_for_form.fields:
        field_default = request.GET.get(field, None)
        if field_default:
            form_for_form.fields[field].initial = field_default

    if request.method == "POST":
        if form_for_form.is_valid():
            entry = form_for_form.save()
            entry.entry_path = request.POST.get("entry_path", "")
            entry.save()

            # Email
            subject = generate_email_subject(form, entry)
            email_headers = {}  # content type specified below
            if form.email_from:
                email_headers.update({'Reply-To':form.email_from})

            # Email to submitter
            # fields aren't included in submitter body to prevent spam
            submitter_body = generate_submitter_email_body(entry, form_for_form)
            email_from = form.email_from or settings.DEFAULT_FROM_EMAIL
            sender = get_setting('site', 'global', 'siteemailnoreplyaddress')
            email_to = form_for_form.email_to()
            if email_to and form.send_email and form.email_text:
                # Send message to the person who submitted the form.
                msg = EmailMessage(subject, submitter_body, sender, [email_to], headers=email_headers)
                msg.content_subtype = 'html'
                msg.send()

            # Email copies to admin
            admin_body = generate_admin_email_body(entry, form_for_form)
            email_from = email_to or email_from # Send from the email entered.
            email_headers.update({'Reply-To':email_from})
            email_copies = [e.strip() for e in form.email_copies.split(",")
                if e.strip()]
            if email_copies:
                # Send message to the email addresses listed in the copies.
                msg = EmailMessage(subject, admin_body, sender, email_copies, headers=email_headers)
                msg.content_subtype = 'html'
                for f in form_for_form.files.values():
                    f.seek(0)
                    msg.attach(f.name, f.read())
                msg.send()

            # payment redirect
            if (form.custom_payment or form.recurring_payment) and entry.pricing:
                # get the pricing's price, custom or otherwise
                price = entry.pricing.price or form_for_form.cleaned_data.get('custom_price')

                if form.recurring_payment:
                    billing_start_dt = datetime.datetime.now()
                    trial_period_start_dt = None
                    trial_period_end_dt = None
                    if entry.pricing.has_trial_period:
                        trial_period_start_dt = datetime.datetime.now()
                        trial_period_end_dt = trial_period_start_dt + datetime.timedelta(1)
                        billing_start_dt = trial_period_end_dt
                    # Create recurring payment
                    rp = RecurringPayment(
                             user=request.user,
                             description=entry.pricing.label,
                             billing_period=entry.pricing.billing_period,
                             billing_start_dt=billing_start_dt,
                             num_days=entry.pricing.num_days,
                             due_sore=entry.pricing.due_sore,
                             payment_amount=price,
                             taxable=entry.pricing.taxable,
                             tax_rate=entry.pricing.tax_rate,
                             has_trial_period=entry.pricing.has_trial_period,
                             trial_period_start_dt=trial_period_start_dt,
                             trial_period_end_dt=trial_period_end_dt,
                             trial_amount=entry.pricing.trial_amount,
                             creator=request.user,
                             creator_username=request.user.username,
                             owner=request.user,
                             owner_username=request.user.username,
                         )
                    rp.save()
                    rp.add_customer_profile()

                    # redirect to recurring payments
                    messages.add_message(request, messages.SUCCESS, 'Successful transaction.')
                    return redirect('recurring_payment.my_accounts')
                else:
                    # create the invoice
                    invoice = make_invoice_for_entry(entry, custom_price=price)
                    # log an event for invoice add

                    EventLog.objects.log(instance=form)

                    # redirect to billing form
                    return redirect('form_entry_payment', invoice.id, invoice.guid)

            # default redirect
            if form.completion_url:
                return redirect(form.completion_url)
            return redirect("form_sent", form.slug)
    # set form's template to default if no template or template doesn't exist
    if not form.template or not template_exists(form.template):
        form.template = "default.html"
    context = {
        "form": form,
        "form_for_form": form_for_form,
        'form_template': form.template,
    }
    return render_to_response(template, context, RequestContext(request))

def form_sent(request, slug, template="forms/form_sent.html"):
    """
    Show the response message.
    """
    published = Form.objects.published(for_user=request.user)
    form = get_object_or_404(published, slug=slug)
    # set form's template to default if no template or template doesn't exist
    if not form.template or not template_exists(form.template):
        form.template = "default.html"
    context = {"form": form, "form_template": form.template}
    return render_to_response(template, context, RequestContext(request))

def form_entry_payment(request, invoice_id, invoice_guid, form_class=BillingForm, template="forms/form_payment.html"):
    """
    Show billing form, update the invoice then proceed to external payment.
    """
    invoice = get_object_or_404(Invoice, pk=invoice_id)

    if not invoice.allow_view_by(request.user, invoice_guid):
        raise Http403

    entry = FormEntry.objects.get(id=invoice.object_id)

    if request.method == "POST":
        form = form_class(request.POST)
        if form.is_valid():
            update_invoice_for_entry(invoice, form)
            # redirect to online payment
            if (entry.payment_method.machine_name).lower() == 'credit-card':
                return redirect('payment.pay_online', invoice.id, invoice.guid)
            # redirect to invoice page
            return redirect('invoice.view', invoice.id, invoice.guid)
    else:
        if request.user.is_authenticated():
            form = form_class(initial={
                        'first_name':request.user.first_name,
                        'last_name':request.user.last_name,
                        'email':request.user.email})
        else:
            form = form_class()
    # set form's template to default if no template or template doesn't exist
    form_template = entry.form.template
    if not form_template or not template_exists(form_template):
        form_template = "default.html"
    EventLog.objects.log(instance=entry)
    return render_to_response(template, {
            'payment_form':form,
            'form':entry.form,
            'form_template': form_template,
        }, context_instance=RequestContext(request))

@login_required
def export(request, template_name="forms/export.html"):
    """Export forms"""

    if not request.user.is_superuser:
        raise Http403

    if request.method == 'POST':
        export_id = run_export_task('forms_builder.forms', 'form', [])
        EventLog.objects.log()
        return redirect('export.status', export_id)

    return render_to_response(template_name, {
    }, context_instance=RequestContext(request))


@login_required
def files(request, id):
    """
    Returns file.  Allows us to handle privacy.

    If default storage is remote:
        We can get data from remote location, convert to file
        object and return a file response.
    """

    import os
    import mimetypes
    from django.http import Http404
    from django.core.files.base import ContentFile
    from django.core.files.storage import default_storage
    from tendenci.core.perms.utils import has_view_perm
    from tendenci.apps.forms_builder.forms.models import FieldEntry

    field = get_object_or_404(FieldEntry, pk=id)
    form = field.field.form

    base_name = os.path.basename(field.value)
    mime_type = mimetypes.guess_type(base_name)[0]

    if not has_view_perm(request.user, 'forms.view_form', form):
        raise Http403

    if not mime_type:
        raise Http404

    if not default_storage.exists(field.value):
        raise Http404

    data = default_storage.open(field.value).read()
    f = ContentFile(data)

    EventLog.objects.log()
    response = HttpResponse(f.read(), mimetype=mime_type)
    response['Content-Disposition'] = 'filename=%s' % base_name
    return response



from datetime import datetime
from haystack import indexes
from haystack import site

from django.utils.html import strip_tags, strip_entities

from tendenci.addons.directories.models import Directory
from tendenci.core.perms.indexes import TendenciBaseSearchIndex
from tendenci.core.categories.models import Category

class DirectoryIndex(TendenciBaseSearchIndex):
    headline = indexes.CharField(model_attr='headline', faceted=True)
    body = indexes.CharField(model_attr='body')
    activation_dt = indexes.DateTimeField(model_attr='activation_dt', null=True)
    expiration_dt = indexes.DateTimeField(model_attr='expiration_dt', null=True)
    syndicate = indexes.BooleanField(model_attr='syndicate')

    # categories
    category = indexes.CharField()
    sub_category = indexes.CharField()

    # RSS fields
    can_syndicate = indexes.BooleanField()
    order = indexes.DateTimeField()

    def prepare_body(self, obj):
        body = obj.body
        body = strip_tags(body)
        body = strip_entities(body)
        return body

    def prepare_category(self, obj):
        category = Category.objects.get_for_object(obj, 'category')
        if category:
            return category.name
        return ''

    def prepare_sub_category(self, obj):
        category = Category.objects.get_for_object(obj, 'sub_category')
        if category:
            return category.name
        return ''

    def prepare_can_syndicate(self, obj):
        args = [
            obj.allow_anonymous_view,
            obj.syndicate,
            obj.status,
            obj.status_detail == 'active',
        ]

        if obj.activation_dt:
            args.append(obj.activation_dt <= datetime.now())

        if obj.expiration_dt:
            args.append(obj.expiration_dt > datetime.now())

        return all(args)

    def prepare_order(self, obj):
        return obj.activation_dt

site.register(Directory, DirectoryIndex)



from tastypie import fields
from tastypie.resources import ModelResource
from tendenci.core.api_tasty.resources import TendenciResource
from tendenci.core.api_tasty.validation import TendenciValidation
from tendenci.core.api_tasty.entities.resources import EntityResource
from tendenci.addons.events.models import Event, Place, Type

class PlaceResource(ModelResource):
    class Meta:
        queryset = Place.objects.all()
        resource_name = 'place'
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]
        
class TypeResource(ModelResource):
    class Meta:
        queryset = Type.objects.all()
        resource_name = 'type'
        list_allowed_methods = ['get',]
        detail_allowed_methods = ['get',]

class EventResource(TendenciResource):
    entity = fields.ForeignKey(EntityResource, 'entity', null=True)
    type = fields.ForeignKey(TypeResource, 'type', null=True)
    place = fields.ForeignKey(PlaceResource, 'place', null=True)
    
    class Meta(TendenciResource.Meta):
        queryset = Event.objects.all()
        resource_name = 'event'
        list_allowed_methods = ['get', 'post']
        detail_allowed_methods = ['get', 'post', 'put', 'delete']



from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from picklefield.fields import PickledObjectField

class ICS(models.Model):
    class Meta:
        verbose_name_plural = 'ics'

    STATUS_CHOICES = (
        ("completed", _(u"Completed")),
        ("pending", _(u"Pending")),
        ("failed", _(u"Failed")),
    )
    app_label = models.CharField(max_length=50)
    model_name = models.CharField(max_length=50)
    user = models.ForeignKey(User)
    status = models.CharField(_(u"status"), max_length=50,
            default="pending", choices=STATUS_CHOICES)
    result = PickledObjectField(null=True, default=None)
    date_created = models.DateTimeField(auto_now_add=True)
    date_done = models.DateTimeField(auto_now=True)
    
    @models.permalink
    def get_absolute_url(self):
        return ("ics.status", [self.app_label, self.model_name])
    
    def __unicode__(self):
        return "ICS for %s %s - %s" % (self.app_label, self.model_name, self.user)



from django.forms.widgets import Select, TextInput
from django.utils.safestring import mark_safe

def _add_name_to_class_attr(name, kwargs_dict):
    if "attrs" in kwargs_dict:
        if "class" in kwargs_dict["attrs"]:
            kwargs_dict["attrs"]["class"] += " %s" % (name.replace("_", "-"), )
        else:
            kwargs_dict["attrs"].update({'class': name.replace("_", "-")})
    else:
        kwargs_dict["attrs"] = {'class': name.replace("_", "-")}

    return kwargs_dict

def _strip_name_attr(widget_string, name):
    return widget_string.replace("name=\"%s\"" % (name,), "")


class NoNameTextInput(TextInput):
    def render(self, name, *args, **kwargs):
        kwargs = _add_name_to_class_attr(name, kwargs)
        return mark_safe(_strip_name_attr(super(NoNameTextInput, self).render(name, *args, **kwargs), name))



from tendenci.core.registry import site
from tendenci.core.registry.base import LogRegistry, lazy_reverse
from tendenci.apps.contacts.models import Contact

class ContactRegistry(LogRegistry):
    # contacts - TEAL/LIME-GREEN base
    event_logs = {
        'contact':{
            'add_with_new_user':('125114', '33CCCC'), # add - contact form submitted / new user added
            'add_with_existing_user':('125115', '0066CC'), # add - contact form submitted / user already exists
        },
    }

site.register(Contact, ContactRegistry)




import datetime, encoder
from django.conf import settings
from django.db.models import Q
from django.template.defaultfilters import slugify
from django.contrib.contenttypes.models import ContentType
from tagging.models import Tag
from tendenci.core.site_settings.utils import get_setting
from tendenci.core.categories.models import Category
from tendenci.apps.pages.models import Page
    

def gen_xml(data):
    """
    Makes use of the encoder for xml to generate the wordpress export.
    """
    xml = encoder.XML()
    xml.write(
        '<rss version="2.0"\n' +
        '   xmlns:excerpt="http://wordpress.org/export/1.0/excerpt/"\n' +
        '   xmlns:content="http://purl.org/rss/1.0/modules/content/"\n' +
        '   xmlns:wfw="http://wellformedweb.org/CommentAPI/"\n' +
        '   xmlns:dc="http://purl.org/dc/elements/1.1/"\n' +
        '   xmlns:wp="http://wordpress.org/export/1.0/"\n' +
        '>'
    )
    xml.open("channel")
    encode_site(xml)
    offset = 0
    if data["pages"]:
        offset = encode_pages(xml, offset)
    if data["articles"]:
        offset = encode_articles(xml, offset)
    if data["news"]:
        offset = encode_news(xml, offset)
    if data["jobs"]:
        offset = encode_jobs(xml, offset)
    if data["events"]:
        offset = encode_events(xml, offset)
    try:
        from tendenci.addons.case_studies import CaseStudy
        if data["case_studies"]:
            offset = encode_casestudies(xml, offset)
    except ImportError:
        pass
    if data["resumes"]:
        offset = encode_resumes(xml, offset)
    xml.close("channel")
    xml.close("rss")
    return xml

def encode_site(xml):
    xml.write("<title>%s</title>"%get_setting('site', 'global', 'sitedisplayname'), depth=1)
    xml.write("<description></description>", depth=1)
    xml.write("<pubDate>%s</pubDate>"%datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"), depth=1)
    xml.write("<language>%s</language>"%get_setting('site', 'global', 'localizationlanguage'), depth=1)
    xml.write("<wp:wxr_version>%s</wp:wxr_version>"%1.0, depth=1)
    # not sure if this is required
    #xml.write("<wp:base_site_url>http://wordpress.com/</wp:base_site_url>", depth=1)
    #xml.write("<wp:base_blog_url>http://sambixly.wordpress.com</wp:base_blog_url>", depth=1)
    encode_categories(xml)
    encode_tags(xml)
    
def encode_categories(xml):
    # since wordpress categories can only have single parents,
    # I will be not be associating the categories with any parents.
    ct_page = ContentType.objects.get_for_model(Page)
    try:
        from tendenci.addons.articles.models import Article
        ct_article = ContentType.objects.get_for_model(Article)
    except ImportError:
        ct_article = None
    try:
        from tendenci.addons.news.models import News
        ct_news = ContentType.objects.get_for_model(News)
    except ImportError:
        ct_news = None
    try:
        from tendenci.addons.jobs.models import Job
        ct_job = ContentType.objects.get_for_model(Job)
    except ImportError:
        ct_job = None
    try:
        from tendenci.addons.events.models import Event
        ct_event = ContentType.objects.get_for_model(Event)
    except ImportError:
        ct_event = None
    try:
        from tendenci.addons.resumes.models import Resume
        ct_resume = ContentType.objects.get_for_model(Resume)
    except ImportError:
        ct_resume = None
    try:
        from tendenci.addons.case_studies.models import CaseStudy
        ct_casestudy = ContentType.objects.get_for_model(CaseStudy)
    except ImportError:
        ct_casestudy = None
    
    #encode tendenci categories
    cats = Category.objects.filter(
        Q(categoryitem_category__content_type=ct_page) or
        Q(categoryitem_category__content_type=ct_article) or
        Q(categoryitem_category__content_type=ct_news) or
        Q(categoryitem_category__content_type=ct_job) or
        Q(categoryitem_category__content_type=ct_event)).distinct()
    for cat in cats:
        xml.open("wp:category", depth=1)
        xml.write("<wp:category_nicename>%s</wp:category_nicename>"%slugify(cat.name), depth=2)
        xml.write("<wp:category_parent></wp:category_parent>", depth=2)
        xml.write("<wp:cat_name><![CDATA[%s]]></wp:cat_name>"%cat.name, depth=2)
        xml.close("wp:category", depth=1)
        
    #encode content types as categories
    types = [ct_article, ct_job, ct_event, ct_news, ct_page, ct_resume, ct_casestudy]
    for type in types:
        if type:
            xml.open("wp:category", depth=1)
            xml.write("<wp:category_nicename>%s</wp:category_nicename>"%slugify(type), depth=2)
            xml.write("<wp:category_parent></wp:category_parent>", depth=2)
            xml.write("<wp:cat_name><![CDATA[%s]]></wp:cat_name>"%type, depth=2)
            xml.close("wp:category", depth=1)

def encode_tags(xml):
    ct_page = ContentType.objects.get_for_model(Page)
    try:
        from tendenci.addons.articles.models import Article
        ct_article = ContentType.objects.get_for_model(Article)
    except ImportError:
        ct_article = None
    try:
        from tendenci.addons.news.models import News
        ct_news = ContentType.objects.get_for_model(News)
    except ImportError:
        ct_news = None
    try:
        from tendenci.addons.jobs.models import Job
        ct_job = ContentType.objects.get_for_model(Job)
    except ImportError:
        ct_job = None
    try:
        from tendenci.addons.events.models import Event
        ct_event = ContentType.objects.get_for_model(Event)
    except ImportError:
        ct_event = None
    try:
        from tendenci.addons.resumes.models import Resume
        ct_resume = ContentType.objects.get_for_model(Resume)
    except ImportError:
        ct_resume = None
    try:
        from tendenci.addons.case_studies.models import CaseStudy
        ct_casestudy = ContentType.objects.get_for_model(CaseStudy)
    except ImportError:
        ct_casestudy = None
    tags = Tag.objects.filter(
        Q(items__content_type__pk=ct_page.pk) or
        Q(items__content_type__pk=ct_article.pk) or
        Q(items__content_type__pk=ct_news.pk) or
        Q(items__content_type__pk=ct_job.pk) or
        Q(items__content_type__pk=ct_event.pk)).distinct()
    for tag in tags:
        xml.open("wp:tag", depth=1)
        xml.write("<wp:tag_slug>%s</wp:tag_slug>"%slugify(tag), depth=2)
        xml.write("<wp:tag_name><![CDATA[%s]]></wp:tag_name>"%tag, depth=2)
        xml.close("wp:tag", depth=1)
        
def encode_item(xml, offset, item, ct, title="", content=""):
    xml.open("item", depth=1)
    xml.write("<title>%s</title>"%title, depth=2)
    xml.write("<pubDate>%s</pubDate>"%item.create_dt.strftime("%a, %d %b %Y %H:%M:%S %z"), depth=2)
    xml.write("<dc:creator><![CDATA[%s]]></dc:creator>"%item.creator_username, depth=2)
    
    #get all the item's categories
    cats = Category.objects.filter(
        categoryitem_category__object_id = item.id,
        categoryitem_category__content_type = ct)
    
    #encode item's categories
    for cat in cats:
        xml.write("<category><![CDATA[%s]]></category>"%cat.name, depth=2)
        xml.write('<category domain="category" nicename="%s"><![CDATA[%s]]></category>' \
            % (slugify(cat.name),cat.name), depth=2)
        
    #encode ct as category
    xml.write("<category><![CDATA[%s]]></category>"%ct, depth=2)
    xml.write('<category domain="category" nicename="%s"><![CDATA[%s]]></category>' \
            % (slugify(ct),ct), depth=2)
    
    # get all the item's tags and encode
    tags = Tag.objects.get_for_object(item)
    for tag in tags:
        xml.write('<category domain="tag"><![CDATA[%s]]></category>'%tag.name, depth=2)
        xml.write('<category domain="tag" nicename="%s"><![CDATA[%s]]></category>' \
            % (slugify(tag.name), tag.name), depth=2)
            
    xml.write("<description></description>", depth=2)
    xml.write("<content:encoded><![CDATA[%s]]></content:encoded>"%content, depth=2)
    xml.write("<excerpt:encoded><![CDATA[]]></excerpt:encoded>", depth=2)
    xml.write("<wp:post_id>%s</wp:post_id>" % offset, depth=2)
    xml.write("<wp:post_date>%s</wp:post_date>"%item.create_dt, depth=2)
    xml.write("<wp:post_date_gmt>%s</wp:post_date_gmt>"%item.create_dt, depth=2)
    xml.write("<wp:comment_status>open</wp:comment_status>", depth=2)
    xml.write("<wp:ping_status>open</wp:ping_status>", depth=2)
    xml.write("<wp:post_name>%s</wp:post_name>"%slugify(title), depth=2)
    xml.write("<wp:status>publish</wp:status>", depth=2)
    xml.write("<wp:post_parent>0</wp:post_parent>", depth=2)
    xml.write("<wp:menu_order>0</wp:menu_order>", depth=2)
    xml.write("<wp:post_type>post</wp:post_type>", depth=2)
    xml.write("<wp:post_password></wp:post_password>", depth=2)
    xml.write("<wp:is_sticky>0</wp:is_sticky>", depth=2)
    xml.close("item", depth=1)
    
def encode_pages(xml, offset=0):
    pages = Page.objects.filter(status=True)
    ct = ContentType.objects.get_for_model(Page)
    for page in pages:
        offset = offset+1
        encode_item(xml, offset, page, ct, title=page.title, content=page.content)
    return offset
        
def encode_articles(xml, offset=0):
    try:
        from tendenci.addons.articles.models import Article
        articles = Article.objects.filter(status=True)
        ct = ContentType.objects.get_for_model(Article)
        for article in articles:
            offset = offset+1
            encode_item(xml, offset, article, ct, title=article.headline, content=article.body)
        return offset
    except ImportError:
        pass
        
def encode_news(xml, offset=0):
    try:
        from tendenci.addons.news.models import News
        newss = News.objects.filter(status=True)
        ct = ContentType.objects.get_for_model(News)
        for news in newss:
            offset = offset+1
            encode_item(xml, offset, news, ct, title=news.headline, content=news.body)
        return offset
    except ImportError:
        pass
        
def encode_jobs(xml, offset=0):
    try:
        from tendenci.addons.jobs.models import Job
        from tendenci.addons.jobs.models import Job
        jobs = Job.objects.filter(status=True)
        ct = ContentType.objects.get_for_model(Job)
        for job in jobs:
            offset = offset+1
            content = job.description
            content += "\n <h2>Location:</h2> %s" % job.location
            content += "\n <h2>Required Experience:</h2> %s" % job.experience
            content += "\n <h2>Required Skills:</h2> %s" % job.skills
            content += "\n <h2>Required Computer Skills:</h2> %s" % job.computer_skills
            content += "\n <h2>Required Education:</h2> %s" % job.education
            encode_item(xml, offset, job, ct, title=job.title, content=content)
    except ImportError:
        pass
    return offset

def encode_events(xml, offset=0):
    try:
        from tendenci.addons.events.models import Event
        events = Event.objects.filter(status=True)
        ct = ContentType.objects.get_for_model(Event)
        for event in events:
            offset = offset+1
            content = "%s to %s\n" % (event.start_dt, event.end_dt)
            content += event.description
            encode_item(xml, offset, event, ct, title=event.title, content=content)
        return offset
    except ImportError:
        pass
        
def encode_resumes(xml, offset=0):
    try:
        from tendenci.addons.resumes.models import Resume
        resumes = Resume.objects.filter(status=True)
        ct = ContentType.objects.get_for_model(Resume)
        for resume in resumes:
            offset = offset+1
            content = resume.description
            content += "\n <h2>Location:</h2> %s" % resume.location
            content += "\n <h2>Experience:</h2> %s" % resume.experience
            content += "\n <h2>Skills:</h2> %s" % resume.skills
            content += "\n <h2>Education:</h2> %s" % resume.education
            content += "\n <h2>Contact Information:</h2>"
            content += "\n %s" % resume.contact_name
            content += "\n %s" % resume.contact_phone
            content += "\n %s" % resume.contact_phone2
            content += "\n %s" % resume.contact_fax
            content += "\n %s" % resume.contact_email
            encode_item(xml, offset, resume, ct, title=resume.title, content=content)
        return offset
    except ImportError:
        pass
        
def encode_casestudies(xml, offset=0):
    try:
        from tendenci.addons.case_studies.models import CaseStudy
        studies = CaseStudy.objects.filter(status=True)
        ct = ContentType.objects.get_for_model(CaseStudy)
        for study in studies:
            offset = offset+1
            content = "<h2>Overview:</h2> \n %s" % study.overview
            if study.execution:
                content += "\n<h2>Execution:</h2> \n %s" % study.execution
            if study.results:
                content += "\n<h2>Results:</h2> \n %s" % study.results
            encode_item(xml, offset, study, ct, title=study.client, content=content)
    except ImportError:
        pass
    return offset



from django.template import RequestContext
from django.shortcuts import render_to_response

from tendenci.apps.accountings.models import Acct

def account_numbers(request, template_name="accountings/account_numbers.html"):
    accts = Acct.objects.all().order_by('account_number')
    return render_to_response(template_name, {'accts': accts, },
                              context_instance=RequestContext(request))



from django.template import Library, TemplateSyntaxError, Variable

from tendenci.core.base.template_tags import ListNode, parse_tag_kwargs
from tendenci.addons.directories.models import Directory

register = Library()

@register.inclusion_tag("directories/options.html", takes_context=True)
def directory_options(context, user, directory):
    context.update({
        "opt_object": directory,
        "user": user
    })
    return context

@register.inclusion_tag("directories/nav.html", takes_context=True)
def directory_nav(context, user, directory=None):
    context.update({
        "nav_object" : directory,
        "user": user
    })
    return context

@register.inclusion_tag("directories/search-form.html", takes_context=True)
def directory_search(context):
    return context


@register.inclusion_tag("directories/pricing-nav.html", takes_context=True)
def directory_pricing_nav(context, user, directory_pricing=None):
    context.update({
        'nav_object': directory_pricing,
        "user": user
    })
    return context

@register.inclusion_tag("directories/pricing-options.html", takes_context=True)
def directory_pricing_options(context, user, directory_pricing):
    context.update({
        "opt_object": directory_pricing,
        "user": user
    })
    return context

@register.inclusion_tag("directories/pricing-table.html", takes_context=True)
def directory_pricing_table(context):
    from tendenci.addons.directories.models import DirectoryPricing
    directory_pricings =DirectoryPricing.objects.filter(status=True).order_by('duration')
    show_premium_price = False
    premium_jp = DirectoryPricing.objects.filter(status=True).filter(premium_price__gt=0)
    if premium_jp:
        show_premium_price = True
    context.update({
        "directory_pricings": directory_pricings,
        'show_premium_price': show_premium_price
    })
    return context


class ListDirectoriesNode(ListNode):
    model = Directory
    perms = 'directories.view_directory'


@register.tag
def list_directories(parser, token):
    """
    Used to pull a list of :model:`directories.Directory` items.

    Usage::

        {% list_directories as [varname] [options] %}

    Be sure the [varname] has a specific name like ``directories_sidebar`` or 
    ``directories_list``. Options can be used as [option]=[value]. Wrap text values
    in quotes like ``tags="cool"``. Options include:
    
        ``limit``
           The number of items that are shown. **Default: 3**
        ``order``
           The order of the items. **Default: Headline A-Z**
        ``user``
           Specify a user to only show public items to all. **Default: Viewing user**
        ``query``
           The text to search for items. Will not affect order.
        ``tags``
           The tags required on items to be included.
        ``random``
           Use this with a value of true to randomize the items included.

    Example::

        {% list_directories as directories_list limit=5 tags="cool" %}
        {% for directory in directories_list %}
            {{ directory.headline }}
        {% endfor %}
    """
    args, kwargs = [], {}
    bits = token.split_contents()
    context_var = bits[2]

    if len(bits) < 3:
        message = "'%s' tag requires more than 3" % bits[0]
        raise TemplateSyntaxError(message)

    if bits[1] != "as":
        message = "'%s' second argument must be 'as" % bits[0]
        raise TemplateSyntaxError(message)

    kwargs = parse_tag_kwargs(bits)

    if 'order' not in kwargs:
        kwargs['order'] = 'headline'

    return ListDirectoriesNode(context_var, *args, **kwargs)



from django.http import HttpResponse
from django.conf import settings
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.core.files.storage import default_storage
from django.core.files.base import ContentFile

from tendenci.core.files.utils import get_image, aspect_ratio, generate_image_cache_key

from tendenci.addons.photos.cache import PHOTO_PRE_KEY
from tendenci.addons.photos.models import Image


def cache_photo_size(id, size, crop=False, quality=90, download=False, constrain=False):
    """
    """
    if isinstance(quality, unicode) and quality.isdigit():
        quality = int(quality)

    cache_key = generate_image_cache_key(file=str(id), size=size, pre_key=PHOTO_PRE_KEY, crop=crop, unique_key=str(id), quality=quality, constrain=constrain)
    cached_image = cache.get(cache_key)
    if cached_image:
        return cached_image

    try:
        photo = Image.objects.get(id=id)
    except:
        return ""

    args = [id, size]
    if crop:
        args.append("crop")
    if constrain:
        args.append("constrain")
    if quality:
        args.append(quality)
    request_path = reverse('photo.size', args=args)

    size = [int(s) for s in size.split('x')]
    size = aspect_ratio(photo.image_dimensions(), size, constrain)

    # gets resized image from cache or rebuild
    image = get_image(photo.image, size, PHOTO_PRE_KEY, crop=crop, quality=quality, unique_key=str(photo.pk), constrain=constrain)

    # if image not rendered; quit
    if not image:
        return request_path

    response = HttpResponse(mimetype='image/jpeg')
    response['Content-Disposition'] = ' filename=%s' % photo.image.file.name
    image.save(response, "JPEG", quality=quality)

    if photo.is_public_photo() and photo.is_public_photoset():
        file_name = photo.image.file.name
        file_path = 'cached%s%s' % (request_path, file_name)
        default_storage.save(file_path, ContentFile(response.content))
        full_file_path = "%s%s" % (settings.MEDIA_URL, file_path)
        cache.set(cache_key, full_file_path)
        cache_group_key = "photos_cache_set.%s" % photo.pk
        cache_group_list = cache.get(cache_group_key)

        if cache_group_list is None:
            cache.set(cache_group_key, [cache_key])
        else:
            cache_group_list += [cache_key]
            cache.set(cache_group_key, cache_group_list)

        return full_file_path
    return request_path
